// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_io.h $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#ifndef __DEV_IO_H__
#define __DEV_IO_H__

#include "gsc_io.h"
#include "gsc_mem.h"

#include "hpdi32_api.h"



// data types *****************************************************************

struct _dev_io_t
{
	gsc_io_t	gio;			// For common I/O use.
	gsc_mem_t	mem;			// For DMA'able memory.
	U32			single_cycle;	// DMDMA: Absent or Present.
	int			use_pio;		// Use PIO vs selected mode.

	struct
	{
		U32						dma_need;	// DMA channel needs.
		const gsc_io_func_t*	mode_funcs;	// When using requested mode.
		const gsc_io_func_t*	pio_funcs;	// When reverting to PIO.
	} cache;
};



// variables ******************************************************************

extern	const gsc_io_func_t	io_pio_read_funcs;
extern	const gsc_io_func_t	io_pio_write_asd_funcs;
extern	const gsc_io_func_t	io_pio_write_ase_funcs;

extern	const gsc_io_func_t	io_dma_read_auto_funcs;
extern	const gsc_io_func_t	io_dma_read_manual_funcs;
extern	const gsc_io_func_t	io_dma_write_auto_ase_funcs;
extern	const gsc_io_func_t	io_dma_write_auto_asd_funcs;
extern	const gsc_io_func_t	io_dma_write_manual_ase_funcs;
extern	const gsc_io_func_t	io_dma_write_manual_asd_funcs;

extern	const gsc_io_func_t	io_dmdma_read_sca_funcs;
extern	const gsc_io_func_t	io_dmdma_read_scd_funcs;
extern	const gsc_io_func_t	io_dmdma_read_scp_funcs;
extern	const gsc_io_func_t	io_dmdma_write_sca_ase_funcs;
extern	const gsc_io_func_t	io_dmdma_write_sca_asd_funcs;
extern	const gsc_io_func_t	io_dmdma_write_scd_ase_funcs;
extern	const gsc_io_func_t	io_dmdma_write_scd_asd_funcs;
extern	const gsc_io_func_t	io_dmdma_write_scp_ase_funcs;
extern	const gsc_io_func_t	io_dmdma_write_scp_asd_funcs;



// prototypes *****************************************************************

// These are primarily for non-configuration parameters.

U32		io_auto_start(gsc_io_t* io);
U32		io_dma_init(dev_data_t* dev);
U32		io_init(dev_data_t* dev, dev_io_t* dio);
U32		io_pio_read_available(dev_data_t* dev, UL* samples);
U32		io_pio_rx_available(gsc_io_t* io);	// For read() services.
U32		io_pio_tx_available(gsc_io_t* io);	// For write() services.
U32		io_pio_write_available(dev_data_t* dev, UL* samples);
void	io_tx_funcs_select(dev_data_t* dev);

// These are primarily for configuration parameters.

U32		io_abort(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_aborted(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_buffer_pointer(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_buffer_size(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_callback_arg(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_callback_func(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_data_size(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_dma_channel_select(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_dma_control_mode(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_dma_priority(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_mode(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_overlap_enable(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_pio_threshold(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_single_cycle(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_status(dev_data_t* dev, U32 which, UL set, UL* get);
U32		io_timeout(dev_data_t* dev, U32 which, UL set, UL* get);



#endif
