// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_io_pio.c $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#include "dev_io.h"

#include "dev_access.h"
#include "dev_irq.h"
#include "dev_reg.h"



// prototypes *****************************************************************

static	U32	_pio_read_wait(gsc_io_t* gio, U32 timeout_ms);
static	U32	_pio_write_wait(gsc_io_t* gio, U32 timeout_ms);



// variables ******************************************************************

// Perform the read without special considerations.
const gsc_io_func_t	io_pio_read_funcs	=
{
	{
	/* _8.setup			*/	NULL,
	/* _8.available		*/	io_pio_rx_available,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	gsc_io_pio_read_8_map,
	/* _8.transfer_reg	*/	gsc_io_pio_read_8_reg,
	/* _8.wait			*/	_pio_read_wait,
	/* _8.done			*/	NULL
	},
	{
	/* _16.setup			*/	NULL,
	/* _16.available	*/	io_pio_rx_available,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	gsc_io_pio_read_16_map,
	/* _16.transfer_reg	*/	gsc_io_pio_read_16_reg,
	/* _16.wait			*/	_pio_read_wait,
	/* _16.done			*/	NULL
	},
	{
	/* _32.setup		*/	NULL,
	/* _32.available	*/	io_pio_rx_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	gsc_io_pio_read_32_map,
	/* _32.transfer_reg	*/	gsc_io_pio_read_32_reg,
	/* _32.wait			*/	_pio_read_wait,
	/* _32.done			*/	NULL
	}
};

// Perform the write with Auto Start disabled.
const gsc_io_func_t	io_pio_write_asd_funcs	=
{
	{
	/* _8.setup			*/	NULL,
	/* _8.available		*/	io_pio_tx_available,
	/* _8.auto_start	*/	NULL,
	/* _8.transfer_map	*/	gsc_io_pio_write_8_map,
	/* _8.transfer_reg	*/	gsc_io_pio_write_8_reg,
	/* _8.wait			*/	_pio_write_wait,
	/* _8.done			*/	NULL
	},
	{
	/* _16.setup		*/	NULL,
	/* _16.available	*/	io_pio_tx_available,
	/* _16.auto_start	*/	NULL,
	/* _16.transfer_map	*/	gsc_io_pio_write_16_map,
	/* _16.transfer_reg	*/	gsc_io_pio_write_16_reg,
	/* _16.wait			*/	_pio_write_wait,
	/* _16.done			*/	NULL
	},
	{
	/* _32.setup		*/	NULL,
	/* _32.available	*/	io_pio_tx_available,
	/* _32.auto_start	*/	NULL,
	/* _32.transfer_map	*/	gsc_io_pio_write_32_map,
	/* _32.transfer_reg	*/	gsc_io_pio_write_32_reg,
	/* _32.wait			*/	_pio_write_wait,
	/* _32.done			*/	NULL
	}
};

// Perform the write with Auto Start enabled.
const gsc_io_func_t	io_pio_write_ase_funcs	=
{
	{
	/* _8.setup			*/	NULL,
	/* _8.available		*/	io_pio_tx_available,
	/* _8.auto_start	*/	io_auto_start,
	/* _8.transfer_map	*/	gsc_io_pio_write_8_map,
	/* _8.transfer_reg	*/	gsc_io_pio_write_8_reg,
	/* _8.wait			*/	_pio_write_wait,
	/* _8.done			*/	NULL
	},
	{
	/* _16.setup		*/	NULL,
	/* _16.available	*/	io_pio_tx_available,
	/* _16.auto_start	*/	io_auto_start,
	/* _16.transfer_map	*/	gsc_io_pio_write_16_map,
	/* _16.transfer_reg	*/	gsc_io_pio_write_16_reg,
	/* _16.wait			*/	_pio_write_wait,
	/* _16.done			*/	NULL
	},
	{
	/* _32.setup		*/	NULL,
	/* _32.available	*/	io_pio_tx_available,
	/* _32.auto_start	*/	io_auto_start,
	/* _32.transfer_map	*/	gsc_io_pio_write_32_map,
	/* _32.transfer_reg	*/	gsc_io_pio_write_32_reg,
	/* _32.wait			*/	_pio_write_wait,
	/* _32.done			*/	NULL
	}
};



/******************************************************************************
*
*	Function:	io_pio_rx_available
*
*	Purpose:
*
*		Determine the number of available bytes, assuming 8-bit, 16-bit or
*		32-bit samples.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 io_pio_rx_available(gsc_io_t* gio)
{
	U32	status;

	status		= io_pio_read_available(gio->dev, &gio->xfer);
	gio->xfer	*= (gio->data_size / 8);
	return(status);
}



/******************************************************************************
*
*	Function:	_pio_read_wait
*
*	Purpose:
*
*		Wait for data to become available to read.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		timeout_ms	The amount of time left to wait.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_read_wait(gsc_io_t* gio, U32 timeout_ms)
{
	#define	_IRQ_	HPDI32_IRQ_RX_E
	U32	status;

	// Wait for the Rx FIFO Not Empty interrupt. However, if we wait too
	// long then I/O aborts will fail due to timeouts.

	status	= irq_interrupt_disable(gio->dev, _IRQ_);

	if (status == (U32) GSC_SUCCESS)
		status	= irq_interrupt_trigger_config(gio->dev, _IRQ_, ~0, 0);

	if (status == (U32) GSC_SUCCESS)
		status	= irq_interrupt_enable(gio->dev, _IRQ_);

	if (status != (U32) GSC_SUCCESS)
	{
	}
	else if (timeout_ms > 100)
	{
		status	= gsc_event_wait(gio->dev->irq.locals[12], 100);

		if (status == (U32) GSC_WAIT_TIMEOUT)
			status	= (U32) GSC_SUCCESS;
	}
	else
	{
		status	= gsc_event_wait(gio->dev->irq.locals[12], timeout_ms);
	}

	return(status);
	#undef	_IRQ_
}



/******************************************************************************
*
*	Function:	io_pio_tx_available
*
*	Purpose:
*
*		Determine the number of available spaces, assuming 8-bit, 16-bit or
*		32-bit samples.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 io_pio_tx_available(gsc_io_t* gio)
{
	U32	status;

	status		= io_pio_write_available(gio->dev, &gio->xfer);
	gio->xfer	*= (gio->data_size / 8);
	return(status);
}



/******************************************************************************
*
*	Function:	_pio_write_wait
*
*	Purpose:
*
*		Wait for write space to become available to write.
*
*	Arguments:
*
*		gio			The common I/O parameter structure.
*
*		timeout_ms	The amount of time left to wait.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

static U32 _pio_write_wait(gsc_io_t* gio, U32 timeout_ms)
{
	#define	_IRQ_	HPDI32_IRQ_TX_F
	U32	status;

	// Wait for the Tx FIFO Not Full interrupt. However, if we wait too
	// long then I/O aborts will fail due to timeouts.

	status	= irq_interrupt_disable(gio->dev, _IRQ_);

	if (status == (U32) GSC_SUCCESS)
		status	= irq_interrupt_trigger_config(gio->dev, _IRQ_, 0, 0);

	if (status == (U32) GSC_SUCCESS)
		status	= irq_interrupt_enable(gio->dev, _IRQ_);

	if (status != (U32) GSC_SUCCESS)
	{
	}
	else if (timeout_ms > 100)
	{
		status	= gsc_event_wait(gio->dev->irq.locals[11], 100);

		if (status == (U32) GSC_WAIT_TIMEOUT)
			status	= (U32) GSC_SUCCESS;
	}
	else
	{
		status	= gsc_event_wait(gio->dev->irq.locals[11], timeout_ms);
	}

	return(status);
	#undef	_IRQ_
}



/******************************************************************************
*
*	Function:	io_pio_read_available
*
*	Purpose:
*
*		Determine the number of 32-bit samples available for reading.
*
*	Arguments:
*
*		dev			The device to access.
*
*		samples		The number of bytes available.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 io_pio_read_available(dev_data_t* dev, UL* samples)
{
	U32	almost;
	U32	bsr;
	U32	rar;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
		if (status != (U32) GSC_SUCCESS)	break;

		if (dev->cache.rfsr)
		{
			// Use the Size register.

			// If the FIFO is full, then the number of
			// samples is the FIFO size.

			if ((bsr & HPDI32_BSR_RX_NOT_FULL) == 0)
			{
				samples[0]	= dev->cache.rfsr;
				break;
			}

			// If the FIFO is Almost Full then there are at
			// least Size - Almost Full samples.

			if ((bsr & HPDI32_BSR_RX_NOT_AF) == 0)
			{
				status	= GSC_REG_READ(dev, HPDI32_RAR, &rar);
				if (status != (U32) GSC_SUCCESS)	break;
				almost		= HPDI32_RAR_DECODE_FULL(rar);
				samples[0]	= dev->cache.rfsr - almost;
				break;
			}

			// If the FIFO is not Almost Empty then there are
			// at least Almost Empty samples.

			if (bsr & HPDI32_BSR_RX_NOT_AE)
			{
				status	= GSC_REG_READ(dev, HPDI32_RAR, &rar);
				if (status != (U32) GSC_SUCCESS)	break;
				samples[0]	= HPDI32_RAR_DECODE_EMPTY(rar);
				break;
			}

			// If the FIFO is not Empty then there is at
			// least one sample.

			if (bsr & HPDI32_BSR_RX_NOT_EMPTY)
			{
				samples[0]	= 1;
				break;
			}

			// The FIFO is empty.

			samples[0]	= 0;
			break;
		}

		//The SIZE registers are not available.

		// If the FIFO is not Almost Empty then there are at
		// least Almost Empty samples.

		if (bsr & HPDI32_BSR_RX_NOT_AE)
		{
			status	= GSC_REG_READ(dev, HPDI32_RAR, &rar);
			if (status != (U32) GSC_SUCCESS)	break;
			samples[0]	= HPDI32_RAR_DECODE_EMPTY(rar);
			break;
		}

		// If the FIFO is not Empty then there is at least one sample.

		if (bsr & HPDI32_BSR_RX_NOT_EMPTY)
		{
			samples[0]	= 1;
			break;
		}

		// The FIFO is empty.

		samples[0]	= 0;
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	io_pio_write_available
*
*	Purpose:
*
*		Determine the number of 32-bit sample spaces available for writing.
*
*	Arguments:
*
*		dev			The device to access.
*
*		samples		The number of bytes available.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 io_pio_write_available(dev_data_t* dev, UL* samples)
{
	U32	ae;
	U32	af;
	U32	bsr;
	U32	status;
	U32	tar;

	for (;;)	// A convenience loop.
	{
		status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
		if (status != (U32) GSC_SUCCESS)	break;

		if (dev->cache.tfsr)
		{
			// Use the Tx FIFO Size in the calculation.

			if ((bsr & HPDI32_BSR_TX_NOT_FULL) == 0)
			{
				// The Tx FIFO is Full.
				// No space available.
				samples[0]	= 0;
				break;
			}

			if ((bsr & HPDI32_BSR_TX_NOT_AF) == 0)
			{
				// The Tx FIFO is Almost Full.
				// At least one space is available.
				samples[0]	= 1;
				break;
			}

			if ((bsr & HPDI32_BSR_TX_NOT_EMPTY) == 0)
			{
				// The Tx FIFO is Empty.
				// The entire Tx FIFO is available.
				samples[0]	= dev->cache.tfsr;
				break;
			}

			if ((bsr & HPDI32_BSR_TX_NOT_AE) == 0)
			{
				// The Tx FIFO is Almost Empty.
				// Available space is at least FIFO size minus Almost Empty.
				status	= GSC_REG_READ(dev, HPDI32_TAR, &tar);
				if (status != (U32) GSC_SUCCESS)	break;
				ae			= HPDI32_TAR_DECODE_EMPTY(tar);
				samples[0]	= dev->cache.tfsr - ae;
				break;
			}

			// The Tx FIFO is between Almost Full and Almost Empty.
			// There is at least Almost Full spaces available.
			status	= GSC_REG_READ(dev, HPDI32_TAR, &tar);
			if (status != (U32) GSC_SUCCESS)	break;
			samples[0]	= HPDI32_TAR_DECODE_FULL(tar);
			break;
		}

		// The Tx FIFO Size Register is unavailable.

		if ((bsr & HPDI32_BSR_TX_NOT_FULL) == 0)
		{
			// The Tx FIFO is Full.
			// No space available.
			samples[0]	= 0;
			break;
		}

		if ((bsr & HPDI32_BSR_TX_NOT_AF) == 0)
		{
			// The Tx FIFO is Almost Full.
			// At least one space is available.
			samples[0]	= 1;
			break;
		}

		if ((bsr & HPDI32_BSR_TX_NOT_EMPTY) == 0)
		{
			// The Tx FIFO is Empty.
			// Available space is at least Almost Full + Almost Empty.
			status	= GSC_REG_READ(dev, HPDI32_TAR, &tar);
			if (status != (U32) GSC_SUCCESS)	break;
			ae			= HPDI32_TAR_DECODE_EMPTY(tar);
			af			= HPDI32_TAR_DECODE_FULL(tar);
			samples[0]	= ae + af;
			break;
		}

		// The Tx FIFO is less than Almost Full.
		// Available space is at least Almost Full.
		status	= GSC_REG_READ(dev, HPDI32_TAR, &tar);
		if (status != (U32) GSC_SUCCESS)	break;
		samples[0]	= HPDI32_TAR_DECODE_FULL(tar);
		break;
	}

	return(status);
}


