// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_reg.c $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#include "dev_reg.h"

#include "gsc_reg.h"



/******************************************************************************
*
*	Function:	hpdi32_reg_mod
*
*	Purpose:
*
*		Perform a read-modify-write on a HPDI32 register.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		reg			The register to read.
*
*		value		The value to write. Excess bits are ignored.
*
*		mask		The set of bits to modify. Excess bits are ignored.
*					If a bit is set here, then the register bits get
*					changed.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 GSC_EXPORT hpdi32_reg_mod(void* handle, U32 reg, U32 value, U32 mask)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (reg & GSC_REG_PCI)
			status	= (U32) GSC_ACCESS_DENIED;
		else if (reg & GSC_REG_PLX)
			status	= (U32) GSC_ACCESS_DENIED;
		else
			status	= gsc_bar_reg_mod(&dev->gsc_regs, reg, value, mask);

		dev_access_unlock(dev);
	}

	return(status);
}



/******************************************************************************
*
*	Function:	hpdi32_reg_read
*
*	Purpose:
*
*		Read a HPDI32 register.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		reg			The register to read.
*
*		value		The value read is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 GSC_EXPORT hpdi32_reg_read(void* handle, U32 reg, U32* value)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (value == NULL)
			status	= (U32) GSC_SUCCESS;
		else if (reg & GSC_REG_PCI)
			status	= gsc_reg_pci_read(&dev->device, reg, value);
		else if (reg & GSC_REG_PLX)
			status	= PLX_REG_READ(dev, reg, value);
		else
			status	= GSC_REG_READ(dev, reg, value);

		dev_access_unlock(dev);
	}

	return(status);
}



/******************************************************************************
*
*	Function:	hpdi32_reg_write
*
*	Purpose:
*
*		Write to a HPDI32 register.
*
*	Arguments:
*
*		handle		The handle of the device to access.
*
*		reg			The register to read.
*
*		value		The value to write. Excess bits are ignored.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 GSC_EXPORT hpdi32_reg_write(void* handle, U32 reg, U32 value)
{
	dev_data_t*	dev;
	U32			status;

	status	= dev_access_lock(handle, &dev);

	if (status == (U32) GSC_SUCCESS)
	{
		if (reg & GSC_REG_PCI)
			status	= (U32) GSC_ACCESS_DENIED;
		else if (reg & GSC_REG_PLX)
			status	= (U32) GSC_ACCESS_DENIED;
		else
			status	= GSC_REG_WRITE(dev, reg, value);

		dev_access_unlock(dev);
	}

	return(status);
}


