// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/dev_tx.c $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#include "dev_tx.h"

#include "dev_config.h"
#include "dev_reg.h"



/******************************************************************************
*
*	Function:	tx_auto_start
*
*	Purpose:
*
*		Implement the Tx Auto Start parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_auto_start(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_AUTO_START_DEFAULT;
		}
		else if (	(set == HPDI32_TX_AUTO_START_NO) ||
					(set == HPDI32_TX_AUTO_START_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_TX_AUTO_START_YES;
		}
		else
		{
			set	= HPDI32_TX_AUTO_START_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == HPDI32_TX_AUTO_START_YES)
			{
				// Auto Start and Remote Throttle operate exclusively.
				status	= tx_remote_throttle(	dev,
												0,
												HPDI32_TX_REMOTE_THROTTLE_DISABLE,
												NULL);
				if (status != (U32) GSC_SUCCESS)	break;
			}

			dev->tx.gio.auto_start	= set;
			io_tx_funcs_select(dev);
		}

		if (get)
			get[0]	= dev->tx.gio.auto_start;

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_auto_stop
*
*	Purpose:
*
*		Implement the Tx Auto Stop parameter service. Validate and apply the
*		setting(s), as appropriate, then retrieve the current setting(s) for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_auto_stop(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_AUTO_STOP_DEFAULT;
		}
		else if (	(set == HPDI32_TX_AUTO_STOP_NO) ||
					(set == HPDI32_TX_AUTO_STOP_YES))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_TX_AUTO_STOP_YES;
		}
		else
		{
			set	= HPDI32_TX_AUTO_STOP_NO;
		}

		if ((set != GSC_NO_CHANGE) && (dev->cache.fsr & HPDI32_FSR_GPIO6_TXAS))
		{
			if (set == HPDI32_TX_AUTO_STOP_YES)
				bcr	= 0;
			else
				bcr	= HPDI32_BCR_TX_AUTO_STOP_NO;

			status	= GSC_REG_MOD(	dev,
									HPDI32_BCR,
									bcr,
									HPDI32_BCR_TX_AUTO_STOP_NO);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			if (dev->cache.fsr & HPDI32_FSR_GPIO6_TXAS)
			{
				get[0]	= HPDI32_TX_AUTO_STOP_YES;
			}
			else
			{
				status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
				if (status != (U32) GSC_SUCCESS)	break;

				if (bcr & HPDI32_BCR_TX_AUTO_STOP_NO)
					get[0]	= HPDI32_TX_AUTO_STOP_NO;
				else
					get[0]	= HPDI32_TX_AUTO_STOP_YES;
			}
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_clock_divider
*
*	Purpose:
*
*		Implement the Tx Clock Divider parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_clock_divider(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;
	U32	tcdr;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_CLOCK_DIVIDER_DEFAULT;
		}
		else if (set <= HPDI32_TX_CLOCK_DIVIDER_MAX)
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	= HPDI32_TX_CLOCK_DIVIDER_MAX;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_WRITE(dev, HPDI32_TCDR, set);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_TCDR, &tcdr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= tcdr & HPDI32_TCDR_DIV_MASK;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_enable
*
*	Purpose:
*
*		Implement the Tx Enable parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_enable(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_ENABLE_DEFAULT;
		}
		else if (	(set == HPDI32_TX_ENABLE_YES) ||
					(set == HPDI32_TX_ENABLE_NO))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_TX_ENABLE_YES;
		}
		else
		{
			set	= HPDI32_TX_ENABLE_NO;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == HPDI32_TX_ENABLE_YES)
				set	= HPDI32_BCR_TX_ENABLED;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, HPDI32_BCR, set, HPDI32_BCR_TX_ENABLED);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bcr & HPDI32_BCR_TX_ENABLED)
					? HPDI32_TX_ENABLE_YES
					: HPDI32_TX_ENABLE_NO;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_flow_control
*
*	Purpose:
*
*		Implement the Tx Flow Control parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_flow_control(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		#if HPDI32_TX_FLOW_CONTROL_IGNORE != GSC_NO_CHANGE
			#error Definition of HPDI32_TX_FLOW_CONTROL_IGNORE no longer GSC_NO_CHANGE.
		#endif

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_FLOW_CONTROL_DEFAULT;
		}
		else if (	(set == HPDI32_TX_FLOW_CONTROL_ENABLE) ||
					(set == HPDI32_TX_FLOW_CONTROL_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_TX_FLOW_CONTROL_ENABLE;
		}
		else
		{
			set	= HPDI32_TX_FLOW_CONTROL_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == HPDI32_TX_FLOW_CONTROL_ENABLE)
				set	= HPDI32_BCR_TX_START;
			else
				set	= 0;

			status	= GSC_REG_MOD(dev, HPDI32_BCR, set, HPDI32_BCR_TX_START);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bcr & HPDI32_BCR_TX_START)
					? HPDI32_TX_FLOW_CONTROL_ENABLE
					: HPDI32_TX_FLOW_CONTROL_DISABLE;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_line_valid_off_count
*
*	Purpose:
*
*		Implement the Tx Line Valid Off parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_line_valid_off_count(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;
	U32	tlilcr;
	U32	tlvlcr;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_TLVLCR, &tlvlcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_LINE_VALID_OFF_COUNT_DEFAULT;
		}
		else if (set <= HPDI32_TX_LINE_VALID_OFF_COUNT_MAX)
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else
		{
			set	= HPDI32_TX_LINE_VALID_OFF_COUNT_MAX;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_WRITE(dev, HPDI32_TLILCR, set);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_TLILCR, &tlilcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= tlilcr & HPDI32_TLILCR_COUNT_MASK;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_line_valid_on_count
*
*	Purpose:
*
*		Implement the Tx Line Valid On parameter service. Validate and apply
*		the setting, as appropriate, then retrieve the current setting for
*		the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_line_valid_on_count(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;
	U32	tlvlcr;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
			;
		else if (set == GSC_DEFAULT)
			set	= HPDI32_TX_LINE_VALID_ON_COUNT_DEFAULT;

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_WRITE(dev, HPDI32_TLVLCR, set);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_TLVLCR, &tlvlcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= tlvlcr;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_overrun
*
*	Purpose:
*
*		Implement the Tx Overrun parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_overrun(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bsr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_OVERRUN_DEFAULT;
		}
		else if (	(set == HPDI32_TX_OVERRUN_CLEAR) ||
					(set == HPDI32_TX_OVERRUN_IGNORE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_TX_OVERRUN_CLEAR;
		}
		else
		{
			set	= HPDI32_TX_OVERRUN_IGNORE;
		}

		if ((set != GSC_NO_CHANGE) &&
			(set != HPDI32_TX_OVERRUN_IGNORE) &&
			(dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN))
		{
			status	= GSC_REG_MOD(	dev,
									HPDI32_BSR,
									HPDI32_BSR_TX_OVERRUN,
									HPDI32_BSR_RX_OVERRUN |
									HPDI32_BSR_RX_UNDER_RUN |
									HPDI32_BSR_TX_OVERRUN);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			if (dev->cache.fsr & HPDI32_FSR_OVR_UNDR_RUN)
			{
				status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
				if (status != (U32) GSC_SUCCESS)	break;

				get[0]	= (bsr & HPDI32_BSR_TX_OVERRUN)
						? HPDI32_TX_OVERRUN_YES
						: HPDI32_TX_OVERRUN_NO;
			}
			else
			{
				get[0]	= HPDI32_TX_OVERRUN_NO;
			}
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_remote_throttle
*
*	Purpose:
*
*		Implement the Tx Remote Throttling parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_remote_throttle(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_REMOTE_THROTTLE_DEFAULT;
		}
		else if (	(set == HPDI32_TX_REMOTE_THROTTLE_ENABLE) ||
					(set == HPDI32_TX_REMOTE_THROTTLE_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_TX_REMOTE_THROTTLE_ENABLE;
		}
		else
		{
			set	= HPDI32_TX_REMOTE_THROTTLE_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == HPDI32_TX_REMOTE_THROTTLE_ENABLE)
			{
				set		= HPDI32_BCR_TX_THROTTLE;
				// Auto Start and Remote Throttle operate exclusively.
				status	= tx_auto_start(dev,
										0,
										HPDI32_TX_AUTO_START_NO,
										NULL);
				if (status != (U32) GSC_SUCCESS)	break;
			}
			else
			{
				set	= 0;
			}

			status	= GSC_REG_MOD(dev, HPDI32_BCR, set, HPDI32_BCR_TX_THROTTLE);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bcr & HPDI32_BCR_TX_THROTTLE)
					? HPDI32_TX_REMOTE_THROTTLE_ENABLE
					: HPDI32_TX_REMOTE_THROTTLE_DISABLE;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_remote_throttle_state
*
*	Purpose:
*
*		Implement the Tx Remote Throttling State parameter service. Validate
*		and apply the setting, as appropriate, then retrieve the current
*		setting for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_remote_throttle_state(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	bsr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported for this parameter.

		if (get)
		{
			if (((bcr & HPDI32_BCR_TX_ENABLED) == 0) ||
				((bcr & HPDI32_BCR_TX_THROTTLE) == 0) ||
				((dev->cache.fsr & HPDI32_FSR_GPIO_0_5) &&
				 ((bcr &  HPDI32_BCR_C3_MASK) != HPDI32_BCR_C3_RR)))
			{
				get[0]	= HPDI32_TX_REMOTE_THROTTLE_STATE_INACTIVE;
			}
			else if (bsr & HPDI32_BSR_RX_READY_)
			{
				get[0]	= HPDI32_TX_REMOTE_THROTTLE_STATE_ACTIVE;
			}
			else
			{
				get[0]	= HPDI32_TX_REMOTE_THROTTLE_STATE_INACTIVE;
			}
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_state
*
*	Purpose:
*
*		Implement the Tx State parameter service. Validate and apply the
*		setting, as appropriate, then retrieve the current setting for the
*		caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_state(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	bsr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BSR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BSR, &bsr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET is not supported for this parameter.

		if (get)
		{
			if ((bcr & HPDI32_BCR_TX_ENABLED) && (bsr & HPDI32_BSR_TX_ACTIVE))
				get[0]	= HPDI32_TX_STATE_ACTIVE;
			else
				get[0]	= HPDI32_TX_STATE_INACTIVE;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_status_valid_count
*
*	Purpose:
*
*		Implement the Tx Status Valid Count parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_status_valid_count(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;
	U32	tsvlcr;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BSR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_STATUS_VALID_COUNT_DEFAULT;
		}

		if (set != GSC_NO_CHANGE)
		{
			status	= GSC_REG_WRITE(dev, HPDI32_TSVLCR, set);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_TSVLCR, &tsvlcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= tsvlcr;
		}

		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_status_valid_mirror
*
*	Purpose:
*
*		Implement the Tx Status Valid Mirror parameter service. Validate and
*		apply the setting, as appropriate, then retrieve the current setting
*		for the caller, also as appropriate.
*
*	Arguments:
*
*		dev			The device to access.
*
*		which		A field of bits specifying the subjects to act on.
*
*		set			The value to apply.
*
*		get			Retriev the current setting an record it here, if not
*					NULL.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		A status reflecting the error encountered.
*
******************************************************************************/

U32 tx_status_valid_mirror(dev_data_t* dev, U32 which, UL set, UL* get)
{
	U32	bcr;
	U32	status;

	for (;;)	// A convenience loop.
	{
		status	= config_which_validate(dev, which, 0);
		if (status != (U32) GSC_SUCCESS)	break;

		status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
		if (status != (U32) GSC_SUCCESS)	break;

		// SET

		if (set == GSC_NO_CHANGE)
		{
		}
		else if (set == GSC_DEFAULT)
		{
			set	= HPDI32_TX_STATUS_VALID_MIRROR_DEFAULT;
		}
		else if (	(set == HPDI32_TX_STATUS_VALID_MIRROR_ENABLE) ||
					(set == HPDI32_TX_STATUS_VALID_MIRROR_DISABLE))
		{
		}
		else if (dev->strict_args)
		{
			status	= (U32) GSC_INVALID_DATA;
			break;
		}
		else if (set)
		{
			set	= HPDI32_TX_STATUS_VALID_MIRROR_ENABLE;
		}
		else
		{
			set	= HPDI32_TX_STATUS_VALID_MIRROR_DISABLE;
		}

		if (set != GSC_NO_CHANGE)
		{
			if (set == HPDI32_TX_STATUS_VALID_MIRROR_ENABLE)
				set	= 0;
			else
				set	= HPDI32_BCR_STATUS_VALID;

			status	= GSC_REG_MOD(dev, HPDI32_BCR, set, HPDI32_BCR_STATUS_VALID);
			if (status != (U32) GSC_SUCCESS)	break;
		}

		if (get)
		{
			status	= GSC_REG_READ(dev, HPDI32_BCR, &bcr);
			if (status != (U32) GSC_SUCCESS)	break;

			get[0]	= (bcr & HPDI32_BCR_STATUS_VALID)
					? HPDI32_TX_STATUS_VALID_MIRROR_DISABLE
					: HPDI32_TX_STATUS_VALID_MIRROR_ENABLE;
		}

		break;
	}

	return(status);
}


