// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_io.h $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#ifndef __GSC_IO_H__
#define __GSC_IO_H__

#include "gsc_access.h"

#include "gsc_bar.h"
#include "gsc_sem.h"

// See additional #includes below.



// data types *****************************************************************

typedef struct _dev_io_t	dev_io_t;	// MUST BE DEFINED BY DEVICE SPECIFIC CODE!
typedef struct _gsc_io_t	gsc_io_t;	// MUST BE DEFINED BY DEVICE SPECIFIC CODE!



#include "gsc_dma.h"



typedef struct
{
	U32	(*setup)(gsc_io_t* io);
	U32	(*available)(gsc_io_t* io);
	U32 (*auto_start)(gsc_io_t* io);
	U32	(*transfer_map)(gsc_io_t* io);
	U32	(*transfer_reg)(gsc_io_t* io);
	U32	(*wait)(gsc_io_t* io, U32 timeout_ms);
	U32	(*done)(gsc_io_t* io);
} gsc_io_func_size_t;

typedef struct
{
	gsc_io_func_size_t	_8;
	gsc_io_func_size_t	_16;
	gsc_io_func_size_t	_32;
} gsc_io_func_t;

typedef struct
{
	U32				enable;		// Yes, No
	gsc_event_t*	transfer;	// Transfer data in background.
} gsc_io_overlap_t;

struct _gsc_io_t	// Most of this is initialize by device specific code.
{
	// These fields are generally static.
	gsc_sem_t			sem;		// Control access.
	gsc_device_t*		device;		// The handle to the device.
	dev_data_t*			dev;		// The device specific structure.
	dev_io_t*			dio;		// The device specific I/O structure.
	const gsc_bar_t*	bar;		// For register based I/O.
	gsc_dma_t*			dma;		// For DMA operations.
	U32					is_tx;		// Is this the Tx side?

	// The GSC PIO code only supports data access that is via a FIFO like
	// register, at this time. The DMA code supports both methods.

	U32					fifo_like;	// Is data accessed like a FIFO?
	U32					reg;		// For register based I/O. fifo_like = 0.
	U32					loc_adrs;	// Used when fifo_like != 0.
	gsc_io_overlap_t	overlap;	// Accessed only by common overlap code.
	gsc_event_t*		user;		// Accessed only by common code.

	// These fields are generally configurable I/O parameters.
	U32					data_lane;		// 0, 8, 16, 24
	U32					data_size;		// 8, 16, 32
	U32					dma_demand;		// Enable (DMDMA), Disable (DMA)
	U32					dma_dynamic;	// Close DMA channel when not needed?
	U32					dma_enable;		// Enable (DMA/DMDMA), Disable (PIO)
	U32					dma_control_mode;	// SUPPORTED BY USR CODE ONLY.
	U32					dma_priority;	// Enable, Disable
	U32					pio_threshold;	// Use PIO if this or less.
	U32					timeout_s;		// Overall timeout limit in seconds.
										// Zero means do not block to wait for
										// more.

	// These fields are filled in with each I/O request.
	U32					abort;			// Abort the I/O operation.
	U32					aborted;		// The I/O operation was aborted.
	U32					auto_start;		// Call "fn_auto_start"?
	void*				buffer;			// Application I/O buffer.
	gsc_dma_channel_t*	dma_channel;	// NULL = not in use.
	U32					dma_need;		// DMA channel needs.
	void*				reg_ptr;		// For memory mapped I/O.
	U32					reg_adj;		// Reg adjusted for byte lane.
	U32					loc_adrs_adj;	// Loc adrs adjusted for byte lane.

	// These fields are filled in with each I/O request
	// and are updated as the request progresses.
	UL						xfer;		// Size of intermediate request.
	void*					xfer_buf;	// Buffer for intermediate request.
	U32						xfer_time_end;	// When are we supposed to stop?
	U32						xfer_tot;	// Size of overall request.
	U32						xfered;		// Result of intermediate request.
	U32						xfered_tot;	// Result of overall request.

	// These are the device specific functions called for each request.
	const gsc_io_func_t*	func;		// Set of device specific routines.

	U32						(*fn_setup)(gsc_io_t* io);
	U32						(*fn_auto_start)(gsc_io_t* io);
	U32						(*fn_available)(gsc_io_t* io);
	U32						(*fn_transfer)(gsc_io_t* io);
	U32						(*fn_wait)(gsc_io_t* io, U32 timeout_ms);
	U32						(*fn_done)(gsc_io_t* io);
};



// prototypes *****************************************************************

U32		gsc_io_abort(gsc_io_t* io);
U32		gsc_io_create(	gsc_io_t*		io,
						const char*		name,
						gsc_device_t*	device,
						dev_data_t*		dev,
						dev_io_t*		dio,
						gsc_bar_t*		bar,
						gsc_dma_t*		dma,
						U32				is_tx);

void	gsc_io_destroy(gsc_io_t* io);
U32		gsc_io_init(gsc_io_t* io);

U32		gsc_io_dma_read(gsc_io_t* io);
U32		gsc_io_dma_wait(gsc_io_t* io, U32 timeout_ms);
U32		gsc_io_dma_write(gsc_io_t* io);
U32		gsc_io_overlap_enable(gsc_io_overlap_t* overlap, UL set, UL* get);
U32		gsc_io_pio_read_8_map(gsc_io_t* io);
U32		gsc_io_pio_read_16_map(gsc_io_t* io);
U32		gsc_io_pio_read_32_map(gsc_io_t* io);
U32		gsc_io_pio_read_8_reg(gsc_io_t* io);
U32		gsc_io_pio_read_16_reg(gsc_io_t* io);
U32		gsc_io_pio_read_32_reg(gsc_io_t* io);
U32		gsc_io_pio_write_8_map(gsc_io_t* io);
U32		gsc_io_pio_write_16_map(gsc_io_t* io);
U32		gsc_io_pio_write_32_map(gsc_io_t* io);
U32		gsc_io_pio_write_8_reg(gsc_io_t* io);
U32		gsc_io_pio_write_16_reg(gsc_io_t* io);
U32		gsc_io_pio_write_32_reg(gsc_io_t* io);
U32		gsc_io_status(gsc_io_t* io, UL* status);
U32		gsc_io_transfer_work(gsc_io_t* gsc);
U32		gsc_io_wait(gsc_sem_t* sem, gsc_io_t* tx, gsc_io_t* rx, U32 timeout_ms);

U32		gsc_io_start_transfer(	gsc_sem_t*	sem,
								gsc_io_t*	io,
								U32			(*function)(dev_data_t*	dev,
														void*		buffer,
														U32			bytes,
														U32*		count),
								void*		buffer,
								U32			bytes,
								U32*		count);

U32		gsc_io_callback_config(	gsc_io_t*		io,
								void*			id,
								void			(*callback)(void* arg1, UL arg2, UL arg3),
								void*			arg1,
								UL				arg2,
								UL				arg3,
								gsc_event_t*	dest);




#endif
