// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_status.c $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#include <string.h>

#include "gsc_status.h"



/******************************************************************************
*
*	Function:	gsc_status_text
*
*	Purpose:
*
*		Return a textual description of the given status value.
*
*	Arguments:
*
*		status		The status of interest.
*
*		text		The text goes here.
*
*		size		The size of the above buffer.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_status_text(U32 status, char* text, size_t size)
{
	const char*	desc;

	switch (status)
	{
		default:								desc	= "Status value not recognized";						break;

		case GSC_SUCCESS:						desc	= "The operation was successful.";						break;
    	case GSC_FAILED:						desc	= "The operation failed.";								break;
    	case GSC_ACCESS_DENIED:					desc	= "Access to the device or resource was denied.";		break;
    	case GSC_DMA_CHANNEL_UNAVAILABLE:		desc	= "A DMA channel was unavailable.";						break;
    	case GSC_DMA_CHANNEL_INVALID:			desc	= "A DMA channel identifier was invalid";				break;
    	case GSC_DMA_CHANNEL_TYPE_ERROR:		desc	= "A DMA channel type was invalid";						break;
    	case GSC_DMA_IN_PROGRESS:				desc	= "A DMA operation was already in progress.";			break;
    	case GSC_DMA_DONE:						desc	= "A DMA operation completed.";							break;
    	case GSC_DMA_PAUSED:					desc	= "A DMA operation was paused.";						break;
    	case GSC_DMA_NOT_PAUSED:				desc	= "A DMA operation was not paused.";					break;
    	case GSC_DMA_CHANNEL_COMMAND_INVALID:	desc	= "A DMA channel command was invalid.";					break;
    	case GSC_DMA_MAN_READY:					desc	= "This status should never occure.";					break;
    	case GSC_DMA_MAN_NOT_READY:				desc	= "This status should never occure.";					break;
    	case GSC_DMA_INVALID_CHANNEL_PRIORITY:	desc	= "A DMA priority option was invalid.";					break;
    	case GSC_DMA_MAN_CORRUPTED:				desc	= "This status should never occure.";					break;
    	case GSC_DMA_INVALID_ELEMENT_INDEX:		desc	= "A DMA element index was invalid.";					break;
    	case GSC_DMA_NO_MORE_ELEMENTS:			desc	= "No more DMA elements were available.";				break;
    	case GSC_DMA_SGL_INVALID:				desc	= "A DMA Scatter/Gather request was invalid.";			break;
    	case GSC_DMA_SGL_QUEUE_FULL:			desc	= "A DMA Scatter/Gather request could not be queued.";	break;
    	case GSC_NULL_PARAM:					desc	= "A argument was NULL.";								break;
    	case GSC_INVALID_BUS_INDEX:				desc	= "A BUS index was invalid.";							break;
    	case GSC_UNSUPPORTED_FUNCTION:			desc	= "A requested feature is unsupported.";				break;
    	case GSC_INVALID_PCI_SPACE:				desc	= "A request accessed an invalid PCI space.";			break;
    	case GSC_INVALID_IOP_SPACE:				desc	= "A request accessed an invalid device space.";		break;
    	case GSC_INVALID_SIZE:					desc	= "An argument's value was invalid.";					break;
    	case GSC_INVALID_ADDRESS:				desc	= "An address was invalid.";							break;
    	case GSC_INVALID_ADDRESS_TYPE:			desc	= "An address type was invalid.";						break;
    	case GSC_INVALID_INDEX:					desc	= "An index value was invalid.";						break;
    	case GSC_MU_NOT_READY:					desc	= "The Message Unit was not ready.";					break;
    	case GSC_MU_FIFO_EMPTY:					desc	= "A Message Unit FIFO was empty.";						break;
    	case GSC_MU_FIFO_FULL:					desc	= "A Message Unit FIFO was full.";						break;
    	case GSC_INVALID_REGISTER:				desc	= "A request accessed an invalid register.";			break;
    	case GSC_DOORBELL_CLEAR_FAILED:			desc	= "A doorbell clear request failed.";					break;
    	case GSC_INVALID_USER_PIN:				desc	= "A User Pin identifier was invalid.";					break;
    	case GSC_INVALID_USER_STATE:			desc	= "A User Pin state was invalid.";						break;
    	case GSC_EEPROM_NOT_PRESENT:			desc	= "The configuration EEPROM was not present.";			break;
    	case GSC_EEPROMP_TYPE_NOT_SUPPORTED:	desc	= "The configuration EEPROM type is unsupported.";		break;
    	case GSC_EEPROM_BLANK:					desc	= "The configuration EEPROM is blank.";					break;
    	case GSC_CONFIG_ACCESS_FAILED:			desc	= "The device could not be accessed.";					break;
    	case GSC_INVALID_DEVICE_INFO:			desc	= "The device information was invalid.";				break;
    	case GSC_NO_ACTIVE_DRIVER:				desc	= "The device driver was not found.";					break;
    	case GSC_INSUFFICIENT_RESOURCES:		desc	= "Insufficient resources.";							break;
    	case GSC_OBJECT_ALREADY_ALLOCATED:		desc	= "The object has already been allocated.";				break;
    	case GSC_ALREADY_INITIALIZED:			desc	= "The device has already been initialized.";			break;
    	case GSC_NOT_INITIALIZED:				desc	= "The device has not been initialized.";				break;
    	case GSC_BAD_CONFIG_REG_ENDIAN_MODE:	desc	= "The Endian mode was incorrect.";						break;
    	case GSC_INVALID_POWER_STATE:			desc	= "The power state option was invalid.";				break;
    	case GSC_POWER_DOWN:					desc	= "The device is powered down.";						break;
    	case GSC_FLYBY_NOT_SUPPORTED:			desc	= "Fly By is not supported.";							break;
    	case GSC_CHANNEL_NOT_SUPPORTED:			desc	= "The channel was not supported.";						break;
    	case GSC_NO_ACTION:						desc	= "No action was requested or performed.";				break;
    	case GSC_HS_NOT_SUPPORTED:				desc	= "Hot Swap is not supported.";							break;
    	case GSC_VP_NOT_SUPPORTED:				desc	= "Vendor Private Data is not supported.";				break;
    	case GSC_VPD_NOT_ENABLED:				desc	= "Vendor Private Data is not enabled.";				break;
    	case GSC_NO_MORE_CAP:					desc	= "No additional capabilities are present.";			break;
    	case GSC_INVALID_OFFSET:				desc	= "An offset value was invalid.";						break;
    	case GSC_BAD_PIN_DIRECTION:				desc	= "A pin direction was invalid.";						break;
    	case GSC_PCI_TIMEOUT:					desc	= "A PCI bus access timed out.";						break;
    	case GSC_DMA_CHANNEL_CLOSED:			desc	= "The DMA channel was closed.";						break;
    	case GSC_DMA_CHANNEL_ERROR:				desc	= "A DMA channel request failed.";						break;
    	case GSC_INVALID_HANDLE:				desc	= "A device handle was invalid.";						break;
    	case GSC_BUFFER_NOT_READY:				desc	= "A buffer was not ready.";							break;
    	case GSC_INVALID_DATA:					desc	= "A data value was invalid.";							break;
    	case GSC_DO_NOTHING:					desc	= "No action was requested or performed.";				break;
    	case GSC_DMA_SGL_BUILD_FAILED:			desc	= "A DMA Scatter/Gather list build failed.";			break;
    	case GSC_PM_NOT_SUPPORTED:				desc	= "Power Management is not supported.";					break;
    	case GSC_INVALID_VERSION_DRIVER:		desc	= "The driver version number is invalid";				break;
    	case GSC_WAIT_TIMEOUT:					desc	= "A wait operation timed out.";						break;
    	case GSC_WAIT_CANCELED:					desc	= "A wait operation was cancelled";						break;

		case GSC_LIB_LOAD_FAILURE:				desc	= "A library load operation failed.";					break;
		case GSC_RESERVED2:						desc	= "This status is no longer generated.";				break;
    	case GSC_INIT_FAILURE:					desc	= "Initialization of the API Library failed.";			break;
    	case GSC_INVALID_VERSION_API:			desc	= "The API Library version number is incorrect";		break;
    	case GSC_RESERVED:						desc	= "This status is no longer generated.";				break;
		case GSC_INTERNAL_ERROR:				desc	= "There was an internal API Library error";			break;
		case GSC_TOO_MANY_OPEN_HANDLES:			desc	= "Too many devices are already open";					break;
		case GSC_INVALID_API_HANDLE:			desc	= "The device handle was invalid";						break;
		case GSC_THREAD_FAILURE:				desc	= "The API could not start a support thread";			break;
		case GSC_ABORTED:						desc	= "The operation was aborted";							break;
	}

	if (text == NULL)
	{
		status	= (U32) GSC_NULL_PARAM;
	}
	else if (size <= 1)
	{
		status	= (U32) GSC_INVALID_SIZE;
	}
	else
	{
		strncpy(text, desc, size);
		text[size - 1]	= 0;
		status	= (U32) GSC_SUCCESS;
	}

	return(status);
}


