// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/api/gsc_version.c $
// $Rev: 30377 $
// $Date: 2014-09-18 12:09:04 -0500 (Thu, 18 Sep 2014) $

#include <string.h>

#include "gsc_version.h"

#include "gsc_drv.h"

#include "dev_version.h"



/******************************************************************************
*
*	Function:	gsc_version_get
*
*	Purpose:
*
*		Provide version data about a specific software component.
*
*	Arguments:
*
*		device		The data used to access the device.
*
*		id			The item whose version number is desired.
*
*		version		The version number string is put here.
*
*		size		This is the size of the above buffer.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_version_get(gsc_device_t* device, U8 id, char* version, size_t size)
{
	U8	api;
	U32	status;

	version[0]	= 0;

	switch (id)
	{
		default:

			status	= (U32) GSC_INVALID_DATA;
			break;

		case GSC_VERSION_LIBRARY:

			strncpy(version, API_GSC_FILE_VERSION_STR, size);
			version[size - 1]	= 0;
			status				= (U32) GSC_SUCCESS;
			break;

		case GSC_VERSION_DRIVER:

			size	= (size > 255) ? 255 : size;
			status	= gsc_drv_version_get(	device,
											&api,
											version,
											(U8) size);
			break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	gsc_version_verify
*
*	Purpose:
*
*		Verify that the versions of things we're working with are sufficient.
*		The driver version is checked only if we've got an open handle.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 gsc_version_verify(void)
{
	gsc_device_t	device;
	int				opn;
	U32				status;

	gsc_drv_version_verify_dev_open(&opn);

	if (opn)
	{
		status	= gsc_open(0, 0, &device, NULL);

		if (status == (U32) GSC_SUCCESS)
		{
			status	= gsc_drv_version_verify(&device);
			gsc_close(&device);
		}
	}
	else
	{
		status	= gsc_drv_version_verify(NULL);
	}


	return(status);
}


