// This software is covered by the GNU GENERAL PUBLIC LICENSE (GPL).
// Any Device - Driver - Linux
// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/driver/ioctl.c $
// $Rev: 23671 $
// $Date: 2013-10-31 16:23:27 -0500 (Thu, 31 Oct 2013) $

#include "main.h"



/* typedefs	**************************************************************/

typedef	int (*dev_ioctl_service_t)(dev_data_t* dev, void* arg);

typedef struct
{
	unsigned long		cmd;
	dev_ioctl_service_t	func;
} dev_ioctl_t;



/* prototypes	**************************************************************/

static int	_version_get_ioctl(dev_data_t*, gsc_drv_version_t* arg);



/* variables	**************************************************************/

/*
*	IOCTL services must be entered into this table to be supported by
*	the driver. The services MUST appear in the table in the position
*	corresponding to its service number (which is encoded in the IOCTL
*	macro.
*/

static const dev_ioctl_t	_ioctl_list[]	=
{
	{
		/* cmd	*/	GSC_IOCTL_VERSION_GET,
		/* func	*/	(dev_ioctl_service_t) _version_get_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_BAR_INFO_GET,
		/* func	*/	(dev_ioctl_service_t) bar_info_get_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_REG_MOD,
		/* func	*/	(dev_ioctl_service_t) reg_mod_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_REG_READ,
		/* func	*/	(dev_ioctl_service_t) reg_read_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_REG_WRITE,
		/* func	*/	(dev_ioctl_service_t) reg_write_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_MEM_ALLOC,
		/* func	*/	(dev_ioctl_service_t) mem_alloc_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_MEM_FREE,
		/* func	*/	(dev_ioctl_service_t) mem_free_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_MEM_INFO,
		/* func	*/	(dev_ioctl_service_t) mem_info_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_IRQ_DISABLE,
		/* func	*/	(dev_ioctl_service_t) irq_disable_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_IRQ_ENABLE,
		/* func	*/	(dev_ioctl_service_t) irq_enable_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_IRQ_NOTIFY,
		/* func	*/	(dev_ioctl_service_t) irq_notify_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_IRQ_STATUS,
		/* func	*/	(dev_ioctl_service_t) irq_status_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_IRQ_WAIT,
		/* func	*/	(dev_ioctl_service_t) irq_wait_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_IRQ_WAIT_CANCEL,
		/* func	*/	(dev_ioctl_service_t) irq_wait_cancel_ioctl
	},

	{
		/* cmd	*/	GSC_IOCTL_DMA_ABORT,
		/* func	*/	(dev_ioctl_service_t) dma_abort_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_DMA_CLOSE,
		/* func	*/	(dev_ioctl_service_t) dma_close_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_DMA_OPEN,
		/* func	*/	(dev_ioctl_service_t) dma_open_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_DMA_STATUS,
		/* func	*/	(dev_ioctl_service_t) dma_status_ioctl
	},
	{
		/* cmd	*/	GSC_IOCTL_DMA_XFER,
		/* func	*/	(dev_ioctl_service_t) dma_transfer_ioctl
	}
};



/******************************************************************************
*
*	Function:	_version_get_ioctl
*
*	Purpose:
*
*		Implement the Get Driver Information IOCTL service.
*
*	Arguments:
*
*		dev	The data for the device of interest.
*
*		arg	The IOCTL service's required argument.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

static int _version_get_ioctl(dev_data_t* dev, gsc_drv_version_t* arg)
{
	arg->status	= GSC_SUCCESS;
	arg->api	= GSC_DRV_INTERFACE_VERSION;
	strcpy(arg->version, DRV_GSC_FILE_VERSION_STR);
	strcpy(arg->name, DRV_DEV_DEVICE_NAME_UPPER);
	return(0);
}



/******************************************************************************
*
*	Function:	_common_ioctl
*
*	Purpose:
*
*		Implement the ioctl() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		filp	This the file structure.
*
*		cmd		The desired IOCTL service.
*
*		arg		The optional service specific argument.
*
*	Returned:
*
*		0		All went well.
*		<0		An error code for the problem seen.
*
******************************************************************************/

static int _common_ioctl(	struct inode*	inode,
							struct file*	filp,
							unsigned int	cmd,
							unsigned long	arg)
{
	union
	{
		gsc_drv_bar_t		bar;
		gsc_drv_dma_t		dma;
		gsc_drv_irq_t		irq;
		gsc_drv_mem_t		mem;
		gsc_drv_reg_t		reg;
		gsc_drv_version_t	version;
	} buf;

	dev_data_t*		dev;
	unsigned long	dir;
	unsigned long	index;
	unsigned long	size;
	int				status;
	int				test;
	unsigned long	type;
	unsigned long	ul;

	for (;;)	/* We'll use a loop for convenience. */
	{
		dev	= dev_locate(inode);

		if (dev == NULL)
		{
			/* The referenced device doesn't exist. */
			status	= -ENODEV;
			break;
		}

		test	= down_interruptible(&dev->sem);

		if (test)
		{
			/* We didn't get the semaphore. */
			status	= -ERESTARTSYS;
			break;
		}

		type	= _IOC_TYPE(cmd);

		if (type != 'g')
		{
			/* The IOCTL code isn't ours. */
			status	= -EINVAL;
			up(&dev->sem);
			break;
		}

		index	= _IOC_NR(cmd);

		if (index >= ARRAY_ELEMENTS(_ioctl_list))
		{
			/* The IOCTL service is unrecognized. */
			status	= -EINVAL;
			up(&dev->sem);
			break;
		}

		if (_ioctl_list[index].func == NULL)
		{
			/* The IOCTL service is unimplemented. */
			status	= -EINVAL;
			up(&dev->sem);
			break;
		}

		size	= _IOC_SIZE(cmd);

		if (size != _IOC_SIZE(_ioctl_list[index].cmd))
		{
			/* The IOCTL data size is invalid. */
			status	= -EINVAL;
			up(&dev->sem);
			break;
		}

		if (size > sizeof(buf))
		{
			// The buffer is too small.
			printk(	"%s: _common_ioctl, line %d:"
					" 'buf' is too small:"
					" is %ld, need %ld\n",
					DRV_DEV_DEVICE_NAME_UPPER,
					__LINE__,
					(long) sizeof(buf),
					(long) size);
			status	= -EFAULT;
			break;
		}

		dir	= _IOC_DIR(cmd);

		if (dir & _IOC_WRITE)
		{
			ul	= access_ok(VERIFY_READ, arg, size);

			if (ul == 0)
			{
				/* We can't read from the user's memory. */
				status	= -EFAULT;
				up(&dev->sem);
				break;
			}

			ul	= copy_from_user(&buf, (void*) arg, size);

			if (ul)
			{
				/* We couldn't read from the user's memory. */
				status	= -EFAULT;
				up(&dev->sem);
				break;
			}
		}

		if (dir & _IOC_READ)
		{
			ul	= access_ok(VERIFY_WRITE, arg, size);

			if (ul == 0)
			{
				/* We can't write to the user's memory. */
				status	= -EFAULT;
				up(&dev->sem);
				break;
			}
		}

		/* Process the IOCTL command. */
		status	= _ioctl_list[index].func(dev, &buf);

		if (status)
		{
			/* The service failed. */
			up(&dev->sem);
			break;
		}

		if (dir & _IOC_READ)
		{
			ul	= copy_to_user((void*) arg, &buf, size);

			if (ul)
			{
				/* We couldn't write to the user's memory. */
				status	= -EFAULT;
				up(&dev->sem);
				break;
			}
		}

		status	= 0;
		up(&dev->sem);
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	ioctl
*
*	Purpose:
*
*		Implement the ioctl() procedure.
*
*	Arguments:
*
*		inode	This is the device inode structure.
*
*		fp		This is the file structure.
*
*		cmd		The desired IOCTL service.
*
*		arg		The optional service specific argument.
*
*	Returned:
*
*		0		All went well.
*		< 0		There was a problem.
*
******************************************************************************/

#if GSC_HAVE_IOCTL_BKL
int ioctl(
	struct inode*	inode,
	struct file*	fp,
	unsigned int	cmd,
	unsigned long	arg)
{
	int	ret;

	ret	= _common_ioctl(inode, fp, cmd, arg);
	return(ret);
}
#endif



/******************************************************************************
*
*	Function:	ioctl_compat
*
*	Purpose:
*
*		Implement the compat_ioctl() method.
*
*	Arguments:
*
*		fp	A pointer to the file structure.
*
*		cmd	The service being requested.
*
*		arg	The service specific argument.
*
*	Returned:
*
*		0	All went well.
*		< 0	There was an error.
*
******************************************************************************/

#if HAVE_COMPAT_IOCTL
long ioctl_compat(struct file* fp, unsigned int cmd, unsigned long arg)
{
	int	ret;

	// No additional locking needed as we use a per device lock.
	ret	= _common_ioctl(fp->f_dentry->d_inode, fp, cmd, arg);
	return(ret);
}
#endif



/******************************************************************************
*
*	Function:	ioctl_unlocked
*
*	Purpose:
*
*		Implement the unlocked_ioctl() method.
*
*	Arguments:
*
*		fp	A pointer to the file structure.
*
*		cmd	The service being requested.
*
*		arg	The service specific argument.
*
*	Returned:
*
*		0	All went well.
*		< 0	There was an error.
*
******************************************************************************/

#if HAVE_UNLOCKED_IOCTL
long ioctl_unlocked(struct file* fp, unsigned int cmd, unsigned long arg)
{
	int	ret;

	// No additional locking needed as we use a per device lock.
	ret	= _common_ioctl(fp->f_dentry->d_inode, fp, cmd, arg);
	return(ret);
}
#endif



/******************************************************************************
*
*	Function:	ioctl_init
*
*	Purpose:
*
*		initialize the IOCTL code.
*
*	Arguments:
*
*		None.
*
*	Returned:
*
*		0	All went well.
*		<0	An error code for the problem seen.
*
******************************************************************************/

int ioctl_init(void)
{
	int	i;
	int	status	= 0;

	for (i = ARRAY_ELEMENTS(_ioctl_list); i-- > 0;)
	{
		if (_IOC_NR(_ioctl_list[i].cmd) != i)
		{
			printk(	"<1>%s: IOCTL list is out of order.\n",
					DRV_DEV_DEVICE_NAME_UPPER);
			status	= -EUCLEAN;
			break;
		}
	}

	return(status);
}



