// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/gpio/din/dio.c $
// $Rev: 30433 $
// $Date: 2014-10-07 14:51:13 -0500 (Tue, 07 Oct 2014) $

#include <stdio.h>
#include <time.h>

#include "main.h"



//*****************************************************************************
static void _show_u8(U8 mask, U8 value)
{
	int	bit		= 0x80;
	int	i;

	for (i = 0; i < 8; i++, bit >>= 1)
	{
		if ((bit & mask) == 0)
			printf("-");
		else if (bit & value)
			printf("1");
		else
			printf("0");
	}
}



//*****************************************************************************
static void _show_input(U8 ctrl_mask, U8 ctrl, U32 data_mask, U32 data)
{
	printf("    ");
	_show_u8(ctrl_mask, ctrl);
	printf("  ");
	_show_u8((U8) ((data_mask >> 24) & 0xFF), (U8) ((data  >> 24) & 0xFF));
	printf(" ");
	_show_u8((U8) ((data_mask >> 16) & 0xFF), (U8) ((data  >> 16) & 0xFF));
	printf(" ");
	_show_u8((U8) ((data_mask >> 8) & 0xFF), (U8) ((data  >> 8) & 0xFF));
	printf(" ");
	_show_u8((U8) (data_mask & 0xFF), (U8) (data  & 0xFF));
}



//*****************************************************************************
int dio_in(void* dev, long seconds)
{
	U8		cap			= 0;
	U8		ctrl		= 0;
	U8		ctrl_last;
	U8		ctrl_mask;
	U32		ctrl_sts;
	U32		data		= 0;
	U32		data_last;
	U32		data_mask;
	U32		data_sts;
	int		errs		= 0;
	int		known;
	time_t	now;
	time_t	t_limit;

	for (;;)	// A convenience loop
	{
		errs	+= dev_gpio_dev_cap_get(dev, -1, 1, &cap);
		errs	+= dev_init(dev, -1, 1);
		errs	+= dev_gpio_config_set(dev, -1, 1, 0xFF, 0, 0, 0);

		label_text("GPIO Input");
		printf(	"%ld seconds%s  (monitor input for changes)\n",
				seconds,
				(seconds == 1) ? "" : "s");

		if (cap & HPDI32_GPIO_CAP_CTRL_0_6)
			ctrl_mask	= 0x7F;
		else if (cap & HPDI32_GPIO_CAP_CTRL_0_5)
			ctrl_mask	= 0x3F;
		else
			ctrl_mask	= 0;

		if (cap & HPDI32_GPIO_CAP_DATA_EXP)
			data_mask	= 0xFFFFFFFF;
		else if (cap & HPDI32_GPIO_CAP_DATA_SIM)
			data_mask	= 0xFFFFFFFF;
		else
			data_mask	= 0;

		if ((ctrl_mask == 0x00) && (data_mask == 0x00000000))
		{
			printf(	"SKIPPED  (No GPIO capabilities.)\n");
			break;
		}

		printf("\n");
		printf("    Control   Data (Hi to Low)\n");
		printf("    76543210  76543210 76543210 76543210 76543210\n");
		printf("    ========  ======== ======== ======== ========\n");

		known		= 0;
		ctrl_last	= 0;
		ctrl_sts	= GSC_SUCCESS;
		data_last	= 0;
		data_sts	= GSC_SUCCESS;
		t_limit		= time(NULL) + seconds;

		for (;;)
		{
			if (ctrl_mask)
				ctrl_sts	= hpdi32_gpio_ctrl_rx(dev, &ctrl);

			if (data_mask)
				data_sts	= hpdi32_gpio_data_rx(dev, &data);

			if ((known == 0) || (ctrl != ctrl_last) || (data != data_last))
			{
				_show_input(ctrl_mask, ctrl, data_mask, data);
				known		= 1;
				ctrl_last	= ctrl;
				data_last	= data;

				if ((ctrl_sts != GSC_SUCCESS) || (data_sts != GSC_SUCCESS))
				{
					printf(	" FAIL <---  (Control: %s, Data: %s)",
							(ctrl_sts == GSC_SUCCESS) ? "PASS" : "FAIL",
							(data_sts == GSC_SUCCESS) ? "PASS" : "FAIL");
				}

				printf("\n");
			}

			now	= time(NULL);

			if (now >= t_limit)
				break;
		}

		printf("\n");
		dev_gpio_dev_release(dev, -1, 1);
		errs	+= dev_init(dev, -1, 1);
		break;
	}

	return(errs);
}


