// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/gpio/din/main.c $
// $Rev: 30433 $
// $Date: 2014-10-07 14:51:13 -0500 (Tue, 07 Oct 2014) $

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "main.h"



// variables ******************************************************************

static	int		_continuous		= 0;
static	int		_def_index		= -1;
static	int		_ignore_errors	= 0;
static	long	_minute_limit	= 0;
static	long	_seconds		= 10;
static	int		_test_limit		= -1;



/******************************************************************************
*
*	Function:	_parse_args
*
*	Purpose:
*
*		Parse the command line arguments.
*
*	Arguments:
*
*		argc	The number of command line arguments given.
*
*		argv	The list of command line arguments given.
*
*	Returned:
*
*		>= 0	The number of errors encounterred.
*
******************************************************************************/

static int _parse_args(int argc, char** argv)
{
	char	buf[32];
	char	c;
	int		errs	= 0;
	int		i;
	int		j;
	int		k;

	printf("din - GPIO Input Sample Application (Version %s)\n", VERSION);
	printf("USAGE: din <-c> <-C> <-m#> <-n#> <-s#> [index]\n");
	printf("  -c     Continue testing until an error occurs.\n");
	printf("  -C     Continue testing even if errors occur.\n");
	printf("  -m#    Run for at most # minutes (a decimal number).\n");
	printf("  -n#    Repeat test at most # times (a decimal number).\n");
	printf("  -s#    Monitor the inputs for this many seconds (a decimal number).\n");
	printf("  index  The zero based index of the device to access.\n");

	for (i = 0; i < argc; i++)
	{
		sprintf(buf, "Argument %d", i);
		label_text(buf);
		printf("%s\n", argv[i]);
	}

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-c") == 0)
		{
			_continuous	= 1;
			_ignore_errors	= 0;
			continue;
		}

		if (strcmp(argv[i], "-C") == 0)
		{
			_continuous	= 1;
			_ignore_errors	= 1;
			continue;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'm') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_minute_limit	= k;
				continue;
			}

			errs	= 1;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			break;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'n') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_test_limit	= k;
				continue;
			}

			errs	= 1;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			break;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 's') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0) && (k <= 3600))
			{
				_seconds	= k;
				continue;
			}

			errs	= 1;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			break;
		}

		j	= sscanf(argv[i], "%d%c", &k, &c);

		if ((j == 1) && (k >= 0))
		{
			_def_index	= k;
			continue;
		}

		errs	= 1;
		printf("ERROR: invalid argument: %s\n", argv[i]);
		break;
	}

	if ((errs == 0) && (_def_index < 0))
	{
		errs	= 1;
		printf("ERROR: Device index not specified.\n");
	}

	return(errs);
}



//*****************************************************************************
static int _perform_tests(void* dev)
{
	int			errs	= 0;
	const char*	psz;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	label_text("Performing Test");
	printf("%s", psz);

	errs	+= id_software(dev, VERSION);
	errs	+= id_hardware(dev, -1);
	errs	+= dio_in(dev, _seconds);

	return(errs);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Control the overall operation of the program.
*
*	Arguments:
*
*		argc			The number of arguments passed to the program.
*
*		argv			The list of arguments passed to the program.
*
*	Returned:
*
*		EXIT_SUCCESS	All went well.
*
******************************************************************************/

int main(int argc, char** argv)
{
	void*	dev			= NULL;
	int		errs		= 0;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty;
	int		ret			= EXIT_FAILURE;
	long	secs;
	time_t	t_limit;
	time_t	test;
	long	tests		= 0;


	for (;;)	// This is a convenience loop.
	{
		test	= time(NULL);
		label_init(25);
		errs	+= _parse_args(argc, argv);

		if (errs)
			break;

		t_limit	= exec + (_minute_limit * 60);
		id_host();
		errs	= dev_api_status();

		if (errs)
			break;

		qty	= dev_board_count();

		if (qty <= 0)
			break;

		label_text("Testing Device Index");
		printf("%d\n", _def_index);
		errs	= dev_open((U8) _def_index, &dev);

		if (errs == 0)
		{
			dev_gpio_lib_init(1);
			ret		= EXIT_SUCCESS;
			errs	= _perform_tests(dev);
			errs	+= dev_close((U8) _def_index, dev);
			dev_gpio_lib_release(1);

			label_text("Status");
			printf("%s\n", errs ? "FAIL <---" : "PASS");
		}

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf(	"\nRESULTS: FAIL <---  (%d error%s)",
					errs,
					(errs == 1) ? "" : "s");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= now - test;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= now - exec;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if ((_test_limit > 0) && (_test_limit <= tests))
			break;

		if (_continuous == 0)
			break;

		if ((_ignore_errors == 0) && (errs))
			break;

		if ((_minute_limit) && (now >= t_limit))
			break;
	}

	return(ret);
}



