// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/gpio/lib/config.c $
// $Rev: 30430 $
// $Date: 2014-10-07 14:49:48 -0500 (Tue, 07 Oct 2014) $

#include <pthread.h>
#include <string.h>

#include "main.h"



// #defines *******************************************************************

#define	CACHE_SIZE		32



// data types *****************************************************************

typedef struct
{
	void*	dev;
	U8		cap;
	U8		ctrl_cfg_mask;
	U8		data_cfg_mask;
} gpio_config_t;		// stores device capabilities



// variables ******************************************************************

static	gpio_config_t	_cache[CACHE_SIZE];
static	int				_init;
static	pthread_mutex_t	_lock;



//*****************************************************************************
static void _config_locate(void* dev, gpio_config_t* config)
{
	int	i;
	U32	status;

	if (config)
	{
		if (_init)
			i	= pthread_mutex_lock(&_lock);
		else
			i	= -1;

		status	= (i == 0) ? GSC_SUCCESS : GSC_FAILED;
		memset(config, 0, sizeof(config[0]));

		for (i = 0; i < CACHE_SIZE; i++)
		{
			if (_cache[i].dev == dev)
			{
				config[0]	= _cache[i];
				break;
			}
		}

		if (status == GSC_SUCCESS)
			pthread_mutex_unlock(&_lock);
	}
}



//*****************************************************************************
static void _config_release(void* dev)
{
	int	i;
	U32	status;

	if (_init)
		i	= pthread_mutex_lock(&_lock);
	else
		i	= -1;

	status	= (i == 0) ? GSC_SUCCESS : GSC_FAILED;

	for (i = 0; i < CACHE_SIZE; i++)
	{
		if (_cache[i].dev == dev)
		{
			_cache[i].dev	= NULL;
			break;
		}
	}

	if (status == GSC_SUCCESS)
		pthread_mutex_unlock(&_lock);
}



//*****************************************************************************
static void _config_save(void* dev, gpio_config_t* config)
{
	int				i;
	U32				status;
	gpio_config_t	tmp;

	if (config)
	{
		_config_locate(dev, &tmp);

		if (tmp.dev != dev)
		{
			if (_init)
				i	= pthread_mutex_lock(&_lock);
			else
				i	= -1;

			status	= (i == 0) ? GSC_SUCCESS : GSC_FAILED;

			for (i = 0; i < CACHE_SIZE; i++)
			{
				if (_cache[i].dev == NULL)
				{
					_cache[i]	= config[0];
					break;
				}
			}

			if (status == GSC_SUCCESS)
				pthread_mutex_unlock(&_lock);
		}
	}
}



//*****************************************************************************
static U32 _dir_out_get(void* dev, U8* cfg_dir)
{
	U8	ctrl_out	= 0;
	U8	data_out	= 0;
	U32	status		= GSC_SUCCESS;
	U32	sts;

	sts		= ctrl_dir_out_get(dev, &ctrl_out);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= data_dir_out_get(dev, &data_out);
	status	= (status == GSC_SUCCESS) ? sts : status;

	if (cfg_dir)
		cfg_dir[0]	= ctrl_out | data_out;

	return(status);
}



//*****************************************************************************
static U32 _dir_out_set(void* dev, int cfg_dir, U8 ctrl_io_bin, U32 data_io_bin)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	sts		= ctrl_dir_out_set(dev, cfg_dir, ctrl_io_bin);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= data_dir_out_set(dev, cfg_dir, data_io_bin);
	status	= (status == GSC_SUCCESS) ? sts : status;
	return(status);
}



//*****************************************************************************
static U32 _enable_get(void* dev, U8* cfg_en)
{
	U8	ctrl_en	= 0;
	U8	data_en	= 0;
	U32	status	= GSC_SUCCESS;
	U32	sts;

	sts		= ctrl_enable_get(dev, &ctrl_en);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= data_enable_get(dev, &data_en);
	status	= (status == GSC_SUCCESS) ? sts : status;

	if (cfg_en)
		cfg_en[0]	= ctrl_en | data_en;

	return(status);
}



//*****************************************************************************
static U32 _enable_set(void* dev, U8 cfg_en, U8 cfg_dir, U8 ctrl_io_bin, U32 data_io_bin)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	sts		= ctrl_enable_set(dev, cfg_en, cfg_dir, ctrl_io_bin);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= data_enable_set(dev, cfg_en, cfg_dir, data_io_bin);
	status	= (status == GSC_SUCCESS) ? sts : status;
	return(status);
}



//*****************************************************************************
U32 gpio_dev_cap_get(void* dev, U8* cap, U8* ctrl_cfg_mask, U8* data_cfg_mask)
{
	gpio_config_t	config;
	U8				ctrl_cap;
	U8				data_cap;
	U32				status	= GSC_SUCCESS;
	U32				sts;

	_config_locate(dev, &config);

	if (config.dev != dev)
	{
		config.dev	= dev;
		ctrl_cap	= 0;
		data_cap	= 0;

		sts		= ctrl_cap_get(dev, &ctrl_cap, &config.ctrl_cfg_mask);
		status	= (status == GSC_SUCCESS) ? sts : status;

		sts		= data_cap_get(dev, &data_cap, &config.data_cfg_mask);
		status	= (status == GSC_SUCCESS) ? sts : status;

		config.cap	= ctrl_cap | data_cap;
		_config_save(dev, &config);
	}

	if (cap)
		cap[0]				= config.cap;

	if (ctrl_cfg_mask)
		ctrl_cfg_mask[0]	= config.ctrl_cfg_mask;

	if (data_cfg_mask)
		data_cfg_mask[0]	= config.data_cfg_mask;

	return(status);
}



//*****************************************************************************
U32 hpdi32_gpio_dev_cap_get(void* dev, U8* cap)
{
	gpio_config_t	config;
	U32				status;

	status	= gpio_dev_cap_get(dev, &config.cap, &config.ctrl_cfg_mask, &config.data_cfg_mask);

	if (cap)
		cap[0]	= config.cap;

	return(status);
}



//*****************************************************************************
U32 hpdi32_gpio_config_get(void* dev, U8* cfg_en, U8* cfg_dir)
{
	U8	cfg_dir_tmp	= 0;
	U8	cfg_en_tmp	= 0;
	U32	status		= GSC_SUCCESS;
	U32	sts;

	sts		= _enable_get(dev, &cfg_en_tmp);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= _dir_out_get(dev, &cfg_dir_tmp);
	status	= (status == GSC_SUCCESS) ? sts : status;

	if (cfg_en)
		cfg_en[0]	= cfg_en_tmp;

	if (cfg_dir)
		cfg_dir[0]	= cfg_dir_tmp;

	return(status);
}



//*****************************************************************************
U32 hpdi32_gpio_config_set(void* dev, U8 cfg_en, U8 cfg_dir, U8 ctrl_io_bin, U32 data_io_bin)
{
	U32	status		= GSC_SUCCESS;
	U32	sts;

	sts		= _enable_set(dev, cfg_en, cfg_dir, ctrl_io_bin, data_io_bin);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= _dir_out_set(dev, cfg_dir, ctrl_io_bin, data_io_bin);
	status	= (status == GSC_SUCCESS) ? sts : status;
	return(status);
}



//*****************************************************************************
void hpdi32_gpio_dev_release(void* dev)
{
	_config_release(dev);
}



//*****************************************************************************
void hpdi32_gpio_lib_init(void)
{
	pthread_mutex_init(&_lock, NULL);
	_init	= 1;
}



//*****************************************************************************
void hpdi32_gpio_lib_release(void)
{
	_init	= 0;
	pthread_mutex_destroy(&_lock);
	memset(&_lock, 0, sizeof(_lock));
}


