// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/gpio/lib/data_exp.c $
// $Rev: 30443 $
// $Date: 2014-10-07 16:31:42 -0500 (Tue, 07 Oct 2014) $

#include <string.h>

#include "main.h"



//*****************************************************************************
U32 data_exp_cap_get(void* dev, U8* cap, U8* cfg_mask)
{
	U8				cap_tmp			= 0;
	U8				cfg_mask_tmp	= 0;
	U32				fsr				= 0;
	U32				status			= GSC_SUCCESS;
	U32				sts;
	unsigned long	supported		= HPDI32_MISC_FEATURES_ABSENT;

	sts		= HPDI32_MISC_FEATURES__FSR(dev, &supported);
	status	= (status == GSC_SUCCESS) ? sts : status;

	sts		= hpdi32_reg_read(dev, HPDI32_FSR, &fsr);
	status	= (status == GSC_SUCCESS) ? sts : status;

	if ((supported == HPDI32_MISC_FEATURES_PRESENT) && (fsr & D9))
	{
		cap_tmp			= HPDI32_GPIO_CAP_DATA_EXP
						| HPDI32_GPIO_CAP_DATA_RX
						| HPDI32_GPIO_CAP_DATA_TX;
		cfg_mask_tmp	= HPDI32_GPIO_CFG_D32;
	}

	if (cap)
		cap[0]		= cap_tmp;

	if (cfg_mask)
		cfg_mask[0]	= cfg_mask_tmp;

	return(status);
}



//*****************************************************************************
U32 data_exp_enable_get(U8* cfg_en)
{
	U32	status	= GSC_SUCCESS;

	// The GPIO registers are supported.
	// Input is always enabled, output may be enabled or disable.

	if (cfg_en)
		cfg_en[0]	= HPDI32_GPIO_CFG_D32;

	return(status);
}



//*****************************************************************************
U32 data_exp_enable_set(void* dev, U8 cfg_en, U8 cfg_dir, U32 io_bin)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	// The GPIO registers are supported.
	// Input is always enabled, output may be enabled or disable.

	if ((cfg_en & HPDI32_GPIO_CFG_D32) == 0)
	{
		// GPIO: disable
		// Tx: disable
		// Rx: always enabled
		sts		= hpdi32_reg_mod(dev, HPDI32_ACR, 0, D0);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}
	else if (cfg_dir & HPDI32_GPIO_CFG_D32)
	{
		// GPIO: enable
		// Tx: enable
		// Rx: always enabled
		sts		= hpdi32_reg_write(dev, HPDI32_GOR, io_bin);
		status	= (status == GSC_SUCCESS) ? sts : status;
		sts		= hpdi32_reg_mod(dev, HPDI32_ACR, D0, D0);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}
	else
	{
		// GPIO: enable
		// Tx: disable
		// Rx: always enabled
		sts		= hpdi32_reg_mod(dev, HPDI32_ACR, 0, D0);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}

	return(status);
}



//*****************************************************************************
U32 data_exp_dir_out_get(void* dev, U8* cfg_dir)
{
	U32	acr;
	U8	cfg_dir_tmp;
	U32	status		= GSC_SUCCESS;
	U32	sts;

	// The GPIO registers are supported.
	// Input is always enabled, output may be enabled or disable.
	sts		= hpdi32_reg_read(dev, HPDI32_ACR, &acr);
	status	= (status == GSC_SUCCESS) ? sts : status;

	if (acr & D0)
		cfg_dir_tmp	= HPDI32_GPIO_CFG_D32;
	else
		cfg_dir_tmp	= 0;

	if (cfg_dir)
		cfg_dir[0]	= cfg_dir_tmp;

	return(status);
}



//*****************************************************************************
U32 data_exp_dir_out_set(void* dev, U8 cfg_dir, U32 io_bin)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	// The GPIO registers are supported.
	// Input is always enabled, output may be enabled or disable.

	if (cfg_dir & HPDI32_GPIO_CFG_D32)
	{
		// GPIO: enable
		// Tx: enable
		// Rx: always enabled
		sts		= hpdi32_reg_write(dev, HPDI32_GOR, io_bin);
		status	= (status == GSC_SUCCESS) ? sts : status;
		sts		= hpdi32_reg_mod(dev, HPDI32_ACR, D0, D0);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}
	else
	{
		// GPIO: enable
		// Tx: disable
		// Rx: always enabled
		sts		= hpdi32_reg_mod(dev, HPDI32_ACR, 0, D0);
		status	= (status == GSC_SUCCESS) ? sts : status;
	}

	return(status);
}



//*****************************************************************************
U32 data_exp_rx(void* dev, U32* io_bin)
{
	U32	io_bin_tmp	= 0;
	U32	status		= GSC_SUCCESS;
	U32	sts;

	// The GPIO registers are supported.
	// Input is always enabled, output may be enabled or disable.
	sts		= hpdi32_reg_read(dev, HPDI32_GIR, &io_bin_tmp);
	status	= (status == GSC_SUCCESS) ? sts : status;

	if (io_bin)
		io_bin[0]	= io_bin_tmp;

	return(status);
}



//*****************************************************************************
U32 data_exp_tx(void* dev, U32 io_bin)
{
	U32	status	= GSC_SUCCESS;
	U32	sts;

	// The GPIO registers are supported.
	// Input is always enabled, output may be enabled or disable.

	sts		= hpdi32_reg_write(dev, HPDI32_GOR, io_bin);
	status	= (status == GSC_SUCCESS) ? sts : status;
	return(status);
}


