// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/sbtest/cable.c $
// $Rev: 30385 $
// $Date: 2014-09-18 12:14:48 -0500 (Thu, 18 Sep 2014) $

#include <stdio.h>
#include <string.h>

#include "main.h"



// data types *****************************************************************

typedef struct
{
	U32	reg;
	U32	value;
	U32	mask;
} reg_t;

typedef struct
{
	U32				parm;
	U32				which;
	unsigned long	set;
	unsigned long	get;
} parm_t;

typedef struct
{
	reg_t	reg;
	parm_t	parm;
} reg_mod_parm_get_t;



/******************************************************************************
*
*	Function:	_reg_mod_config_test
*
*	Purpose:
*
*		Perform a single register-mod-parameter-get test using the service
*		call specified.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		index	The index associated with the below data.
*
*		data	The data used to perform the test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _reg_mod_config_test(void*						handle,
								int							index,
								const reg_mod_parm_get_t*	data)
{
	int				errs;
	unsigned long	get		= 0;
	U32				status;

	status	= hpdi32_reg_mod(	handle,
								data->reg.reg,
								data->reg.value,
								data->reg.mask);

	if (status == GSC_SUCCESS)
	{
		status		= hpdi32_config(handle,
									data->parm.parm,
									data->parm.which,
									data->parm.set,
									&get);
	}

	if (status != GSC_SUCCESS)
	{
		printf("(list index %d) ", index);
		errs	= 1;
	}
	else if (get == data->parm.get)
	{
		errs	= 0;
	}
	else
	{
		printf(	"(%d. set 0x%lX, want 0x%lX, got 0x%lX) ",
				index,
				data->parm.set,
				data->parm.get,
				get);
		errs	= 1;
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_reg_mod_config_list_test
*
*	Purpose:
*
*		Perform a series of register-mod-parameter-get tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		list	The data to apply to the test.
*
*		size	The size of the data set.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _reg_mod_config_list_test(	void*						handle,
										const reg_mod_parm_get_t*	list,
										size_t						size)
{
	int	errs	= 0;
	int	i;

	for (i = 0; (errs == 0) && (i < (int) size); i++)
		errs	= _reg_mod_config_test(handle, i, list + i);

	return(errs);
}



/******************************************************************************
*
*	Function:	_config_state_test
*
*	Purpose:
*
*		Perform a Cable Configuration State test.
*
*	Arguments:
*
*		title	The title of the test.
*
*		handle	The handle used to access the device under test.
*
*		list	The parameter test data to process.
*
*		size	The size of the list.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _config_state_test(	const char*					title,
								void*						handle,
								const reg_mod_parm_get_t*	list,
								size_t						size)
{

	int	errs;

	label_text(title);
	hpdi32_init(handle);
	errs	= _reg_mod_config_list_test(handle, list, size);
	hpdi32_init(handle);

	if (errs)
		printf("FAIL <---\n");
	else
		printf("PASS\n");

	return(errs);
}



/******************************************************************************
*
*	Function:	_clock_state_test
*
*	Purpose:
*
*		Verify the operation of the Clock State parameter.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _clock_state_test(void* handle)
{
	static const reg_mod_parm_get_t	list[]	=
	{
		{
			// Tx Disabled + Rx Disabled = Clock State Inactive
			{
				/* reg.reg		*/	HPDI32_BCR,
				/* reg.value	*/	0,
				/* reg.mask		*/	0xFFFFFFFF
			},
			{
				/* parm.parm	*/	HPDI32_CABLE_CLOCK_STATE,
				/* parm.which	*/	0,
				/* parm.set		*/	GSC_NO_CHANGE,
				/* parm.get		*/	HPDI32_CABLE_CLOCK_STATE_INACTIVE
			}
		},
		{
			// Tx Enabled + Rx Disabled = Clock State Active
			{
				/* reg.reg		*/	HPDI32_BCR,
				/* reg.value	*/	HPDI32_BCR_TX_ENABLED,
				/* reg.mask		*/	0xFFFFFFFF
			},
			{
				/* parm.parm	*/	HPDI32_CABLE_CLOCK_STATE,
				/* parm.which	*/	0,
				/* parm.set		*/	GSC_NO_CHANGE,
				/* parm.get		*/	HPDI32_CABLE_CLOCK_STATE_ACTIVE
			}
		},
		{
			// Tx Disabled + Rx Enabled = Clock State Active
			{
				/* reg.reg		*/	HPDI32_BCR,
				/* reg.value	*/	HPDI32_BCR_RX_ENABLED,
				/* reg.mask		*/	0xFFFFFFFF
			},
			{
				/* parm.parm	*/	HPDI32_CABLE_CLOCK_STATE,
				/* parm.which	*/	0,
				/* parm.set		*/	GSC_NO_CHANGE,
				/* parm.get		*/	HPDI32_CABLE_CLOCK_STATE_ACTIVE
			}
		},
		{
			// Tx Enabled + Rx Enabled = Clock State Active
			{
				/* reg.reg		*/	HPDI32_BCR,
				/* reg.value	*/	HPDI32_BCR_TX_ENABLED |
									HPDI32_BCR_RX_ENABLED |
									HPDI32_BCR_TEST_MODE,
				/* reg.mask		*/	0xFFFFFFFF
			},
			{
				/* parm.parm	*/	HPDI32_CABLE_CLOCK_STATE,
				/* parm.which	*/	0,
				/* parm.set		*/	GSC_NO_CHANGE,
				/* parm.get		*/	HPDI32_CABLE_CLOCK_STATE_ACTIVE
			}
		}
	};

	int			errs;
	const char*	name	= "Clock State";

	errs	= _config_state_test(	name,
									handle,
									list,
									ARRAY_ELEMENTS(list));
	return(errs);
}



/******************************************************************************
*
*	Function:	_flow_control_state_test
*
*	Purpose:
*
*		Verify the operation of a Flow Control State parameter.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*		name	The name of the Flow Control functionality.
*
*		which	The "which" bit for the Cable Command being tested.
*
*		tx		Is the signal active when Tx is enabled?
*
*		rx		Is the signal active when Rx is enabled?
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _flow_control_state_test(
	void*		handle,
	const char*	name,
	U32			which,
	int			tx,
	int			rx)
{
	int					errs;
	char				label[64];
	reg_mod_parm_get_t	list[4];

	sprintf(label, "%s State", name);

	// Tx Disabled + Rx Disabled = Frame Valid State Inactive
	list[0].reg.reg		= HPDI32_BCR;
	list[0].reg.value	= 0;
	list[0].reg.mask	= 0xFFFFFFFF;
	list[0].parm.parm	= HPDI32_CABLE_COMMAND_STATE;
	list[0].parm.which	= which;
	list[0].parm.set	= GSC_NO_CHANGE;
	list[0].parm.get	= HPDI32_CABLE_COMMAND_STATE_INACTIVE;

	// Tx Enabled + Rx Disabled = Frame Valid State Active
	list[1].reg.reg		= HPDI32_BCR;
	list[1].reg.value	= HPDI32_BCR_TX_ENABLED;
	list[1].reg.mask	= 0xFFFFFFFF;
	list[1].parm.parm	= HPDI32_CABLE_COMMAND_STATE;
	list[1].parm.which	= which;
	list[1].parm.set	= GSC_NO_CHANGE;
	list[1].parm.get	= tx
						? HPDI32_CABLE_COMMAND_STATE_ACTIVE
						: HPDI32_CABLE_COMMAND_STATE_INACTIVE;

	// Tx Disabled + Rx Enabled = Frame Valid State Active
	list[2].reg.reg		= HPDI32_BCR;
	list[2].reg.value	= HPDI32_BCR_RX_ENABLED;
	list[2].reg.mask	= 0xFFFFFFFF;
	list[2].parm.parm	= HPDI32_CABLE_COMMAND_STATE;
	list[2].parm.which	= which;
	list[2].parm.set	= GSC_NO_CHANGE;
	list[2].parm.get	= rx
						? HPDI32_CABLE_COMMAND_STATE_ACTIVE
						: HPDI32_CABLE_COMMAND_STATE_INACTIVE;

	// Tx Enabled + Rx Enabled = Frame Valid State Active
	list[3].reg.reg		= HPDI32_BCR;
	list[3].reg.value	= HPDI32_BCR_TX_ENABLED
						| HPDI32_BCR_RX_ENABLED;
	list[3].reg.mask	= 0xFFFFFFFF;
	list[3].parm.parm	= HPDI32_CABLE_COMMAND_STATE;
	list[3].parm.which	= which;
	list[3].parm.set	= GSC_NO_CHANGE;
	list[3].parm.get	= (tx || rx)
						? HPDI32_CABLE_COMMAND_STATE_ACTIVE
						: HPDI32_CABLE_COMMAND_STATE_INACTIVE;

	errs	= _config_state_test(	label,
									handle,
									list,
									ARRAY_ELEMENTS(list));
	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_mode_test
*
*	Purpose:
*
*		Verify the operation of the Cable Command Mode Parameter.
*
*	Arguments:
*
*		handle			The handle used to access the device under test.
*
*		which			The "which" bit for the cable command signal to access.
*
*		feature			Perform the test only if this feature is available.
*
*		fc				This is the BCR value to request Flow Control Mode.
*
*		input			This is the BCR value to request Flow Control Mode.
*
*		output_low		This is the BCR value to request Flow Control Mode.
*
*		output_high		This is the BCR value to request Flow Control Mode.
*
*	Returned:
*
*		0				All went well.
*		> 0				The number of errors seen.
*
******************************************************************************/

static int _cable_command_mode_test(
	void*	handle,
	U32		which,
	U32		feature,
	U32		fc,
	U32		input,
	U32		output_low,
	U32		output_high)
{
	U32					bsr		= 0;
	int					errs;
	unsigned long		get;
	reg_mod_parm_get_t	list[4];
	const char*			name	= "Mode";
	U32					status;

	// Set mode to "Flow Control".
	list[0].reg.reg		= HPDI32_BCR;
	list[0].reg.value	= fc;
	list[0].reg.mask	= 0xFFFFFFFF;
	list[0].parm.parm	= HPDI32_CABLE_COMMAND_MODE;
	list[0].parm.which	= which;
	list[0].parm.set	= GSC_NO_CHANGE;
	list[0].parm.get	= HPDI32_CABLE_COMMAND_MODE_FLOW_CONTROL;

	// Set mode to "GPIO Input".
	list[1].reg.reg		= HPDI32_BCR;
	list[1].reg.value	= input;
	list[1].reg.mask	= 0xFFFFFFFF;
	list[1].parm.parm	= HPDI32_CABLE_COMMAND_MODE;
	list[1].parm.which	= which;
	list[1].parm.set	= GSC_NO_CHANGE;
	list[1].parm.get	= HPDI32_CABLE_COMMAND_MODE_GPIO_IN;

	// Set mode to "GPIO Output Low".
	list[2].reg.reg		= HPDI32_BCR;
	list[2].reg.value	= output_low;
	list[2].reg.mask	= 0xFFFFFFFF;
	list[2].parm.parm	= HPDI32_CABLE_COMMAND_MODE;
	list[2].parm.which	= which;
	list[2].parm.set	= GSC_NO_CHANGE;
	list[2].parm.get	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_LOW;

	// Set mode to "GPIO Output High".
	list[3].reg.reg		= HPDI32_BCR;
	list[3].reg.value	= output_high;
	list[3].reg.mask	= 0xFFFFFFFF;
	list[3].parm.parm	= HPDI32_CABLE_COMMAND_MODE;
	list[3].parm.which	= which;
	list[3].parm.set	= GSC_NO_CHANGE;
	list[3].parm.get	= HPDI32_CABLE_COMMAND_MODE_GPIO_OUT_HI;

	status	= HPDI32_MISC_FEATURES__GET(handle, feature, &get);

	if (status == GSC_SUCCESS)
		status	= hpdi32_reg_read(handle, HPDI32_BSR, &bsr);

	if (status != GSC_SUCCESS)
	{
		errs	= 1;
		label_text(name);
		printf("FAIL <---\n");
	}
	else if (get != HPDI32_MISC_FEATURES_PRESENT)
	{
		errs	= 0;
		label_text(name);
		printf("(GPIO unavailable)\n");
	}
	else if (bsr & HPDI32_BSR_PECL)
	{
		errs	= 0;
		label_text(name);
		printf("SKIPPED (PECL limitations)\n");
	}
	else
	{
		errs	= _config_state_test(	name,
										handle,
										list,
										ARRAY_ELEMENTS(list));
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_gpio_state_test
*
*	Purpose:
*
*		Verify the operation of the Cable Command State Parameter for GPIO
*		configurations.
*
*	Arguments:
*
*		handle			The handle used to access the device under test.
*
*		which			The "which" bit for the cable command signal to access.
*
*		index			The GPIO index number.
*
*		feature			Perform the test only if this feature is available.
*
*		output_low		This is the BCR value to request Flow Control Mode.
*
*		output_high		This is the BCR value to request Flow Control Mode.
*
*	Returned:
*
*		0				All went well.
*		> 0				The number of errors seen.
*
******************************************************************************/

static int _gpio_state_test(
	void*	handle,
	U32		which,
	int		index,
	U32		feature,
	U32		output_low,
	U32		output_high)
{
	U32					bsr		= 0;
	int					errs;
	unsigned long		get;
	char				label[64];
	reg_mod_parm_get_t	list[2];
	U32					status;

	sprintf(label, "GPIO %d", index);

	// Set mode to "GPIO Output Low".
	list[0].reg.reg		= HPDI32_BCR;
	list[0].reg.value	= output_low;
	list[0].reg.mask	= 0xFFFFFFFF;
	list[0].parm.parm	= HPDI32_CABLE_COMMAND_STATE;
	list[0].parm.which	= which;
	list[0].parm.set	= GSC_NO_CHANGE;
	list[0].parm.get	= HPDI32_CABLE_COMMAND_STATE_INACTIVE;

	// Set mode to "GPIO Output High".
	list[1].reg.reg		= HPDI32_BCR;
	list[1].reg.value	= output_high;
	list[1].reg.mask	= 0xFFFFFFFF;
	list[1].parm.parm	= HPDI32_CABLE_COMMAND_STATE;
	list[1].parm.which	= which;
	list[1].parm.set	= GSC_NO_CHANGE;
	list[1].parm.get	= HPDI32_CABLE_COMMAND_STATE_ACTIVE;

	status	= HPDI32_MISC_FEATURES__GET(handle, feature, &get);

	if (status == GSC_SUCCESS)
		status	= hpdi32_reg_read(handle, HPDI32_BSR, &bsr);

	if (status != GSC_SUCCESS)
	{
		errs	= 1;
		label_text(label);
		printf("FAIL <---\n");
	}
	else if (get != HPDI32_MISC_FEATURES_PRESENT)
	{
		errs	= 0;
		label_text(label);
		printf("(GPIO unavailable)\n");
	}
	else if (bsr & HPDI32_BSR_PECL)
	{
		errs	= 0;
		label_text(label);
		printf("SKIPPED (PECL limitations)\n");
	}
	else
	{
		errs	= _config_state_test(	label,
										handle,
										list,
										ARRAY_ELEMENTS(list));
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_0_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _cable_command_0_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Command 0");
	printf("\n");
	label_level_inc();
	errs	+= _cable_command_mode_test(handle,
										HPDI32_WHICH_COMMAND_0_,
										HPDI32_MISC_FEATURES_GPIO_6,
										HPDI32_BCR_C0_FV,
										HPDI32_BCR_C0_IN,
										HPDI32_BCR_C0_OUT_L,
										HPDI32_BCR_C0_OUT_H);
	errs	+= _flow_control_state_test(handle,
										"Frame Valid",
										HPDI32_WHICH_COMMAND_0_,
										TRUE,
										TRUE);
	errs	+= _gpio_state_test(		handle,
										HPDI32_WHICH_COMMAND_0_,
										6,
										HPDI32_MISC_FEATURES_GPIO_6,
										HPDI32_BCR_C0_OUT_L,
										HPDI32_BCR_C0_OUT_H);
	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_1_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _cable_command_1_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Command 1");
	printf("\n");
	label_level_inc();
	errs	+= _cable_command_mode_test(handle,
										HPDI32_WHICH_COMMAND_1_,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C1_LV,
										HPDI32_BCR_C1_IN,
										HPDI32_BCR_C1_OUT_L,
										HPDI32_BCR_C1_OUT_H);
	errs	+= _flow_control_state_test(handle,
										"Line Valid",
										HPDI32_WHICH_COMMAND_1_,
										TRUE,
										TRUE);
	errs	+= _gpio_state_test(		handle,
										HPDI32_WHICH_COMMAND_1_,
										0,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C1_OUT_L,
										HPDI32_BCR_C1_OUT_H);
	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_2_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _cable_command_2_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Command 2");
	printf("\n");
	label_level_inc();
	errs	+= _cable_command_mode_test(handle,
										HPDI32_WHICH_COMMAND_2_,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C2_SV,
										HPDI32_BCR_C2_IN,
										HPDI32_BCR_C2_OUT_L,
										HPDI32_BCR_C2_OUT_H);
	errs	+= _flow_control_state_test(handle,
										"Status Valid",
										HPDI32_WHICH_COMMAND_2_,
										TRUE,
										TRUE);
	errs	+= _gpio_state_test(		handle,
										HPDI32_WHICH_COMMAND_2_,
										1,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C2_OUT_L,
										HPDI32_BCR_C2_OUT_H);
	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_3_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _cable_command_3_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Command 3");
	printf("\n");
	label_level_inc();
	errs	+= _cable_command_mode_test(handle,
										HPDI32_WHICH_COMMAND_3_,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C3_RR,
										HPDI32_BCR_C3_IN,
										HPDI32_BCR_C3_OUT_L,
										HPDI32_BCR_C3_OUT_H);
	errs	+= _flow_control_state_test(handle,
										"Rx Ready",
										HPDI32_WHICH_COMMAND_3_,
										TRUE,
										TRUE);
	errs	+= _gpio_state_test(		handle,
										HPDI32_WHICH_COMMAND_3_,
										2,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C3_OUT_L,
										HPDI32_BCR_C3_OUT_H);
	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_4_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _cable_command_4_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Command 4");
	printf("\n");
	label_level_inc();
	errs	+= _cable_command_mode_test(handle,
										HPDI32_WHICH_COMMAND_4_,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C4_TR,
										HPDI32_BCR_C4_IN,
										HPDI32_BCR_C4_OUT_L,
										HPDI32_BCR_C4_OUT_H);
	errs	+= _flow_control_state_test(handle,
										"Tx Ready",
										HPDI32_WHICH_COMMAND_4_,
										TRUE,
										TRUE);
	errs	+= _gpio_state_test(		handle,
										HPDI32_WHICH_COMMAND_4_,
										3,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C4_OUT_L,
										HPDI32_BCR_C4_OUT_H);
	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_5_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _cable_command_5_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Command 5");
	printf("\n");
	label_level_inc();
	errs	+= _cable_command_mode_test(handle,
										HPDI32_WHICH_COMMAND_5_,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C5_TE,
										HPDI32_BCR_C5_IN,
										HPDI32_BCR_C5_OUT_L,
										HPDI32_BCR_C5_OUT_H);
	errs	+= _flow_control_state_test(handle,
										"Tx Enabled",
										HPDI32_WHICH_COMMAND_5_,
										TRUE,
										FALSE);
	errs	+= _gpio_state_test(		handle,
										HPDI32_WHICH_COMMAND_5_,
										4,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C5_OUT_L,
										HPDI32_BCR_C5_OUT_H);
	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	_cable_command_6_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

static int _cable_command_6_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Command 6");
	printf("\n");
	label_level_inc();
	errs	+= _cable_command_mode_test(handle,
										HPDI32_WHICH_COMMAND_6_,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C6_RE,
										HPDI32_BCR_C6_IN,
										HPDI32_BCR_C6_OUT_L,
										HPDI32_BCR_C6_OUT_H);
	errs	+= _flow_control_state_test(handle,
										"Rx Enabled",
										HPDI32_WHICH_COMMAND_6_,
										FALSE,
										TRUE);
	errs	+= _gpio_state_test(		handle,
										HPDI32_WHICH_COMMAND_6_,
										5,
										HPDI32_MISC_FEATURES_GPIO_0_5,
										HPDI32_BCR_C6_OUT_L,
										HPDI32_BCR_C6_OUT_H);
	label_level_dec();
	return(errs);
}



/******************************************************************************
*
*	Function:	cable_config_tests
*
*	Purpose:
*
*		Perform cable configuration tests.
*
*	Arguments:
*
*		handle	The handle used to access the device under test.
*
*	Returned:
*
*		0		All went well.
*		> 0		The number of errors seen.
*
******************************************************************************/

int cable_config_tests(void* handle)
{
	int	errs	= 0;

	label_text("Cable Configuration");
	printf("\n");
	label_level_inc();

	errs	+= _clock_state_test(handle);
	errs	+= _cable_command_0_tests(handle);
	errs	+= _cable_command_1_tests(handle);
	errs	+= _cable_command_2_tests(handle);
	errs	+= _cable_command_3_tests(handle);
	errs	+= _cable_command_4_tests(handle);
	errs	+= _cable_command_5_tests(handle);
	errs	+= _cable_command_6_tests(handle);

	label_level_dec();
	return(errs);
}


