// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/utils/util_reg.c $
// $Rev: 30444 $
// $Date: 2014-10-07 16:32:08 -0500 (Tue, 07 Oct 2014) $

#include <stdio.h>

#include "hpdi32_utils.h"



// #defines *******************************************************************

#define	_GSC_REG(a)	"GSC " #a, HPDI32_##a
#define	_PCI_REG(a)	"PCI " #a, GSC_PCI_9080_##a
#define	_PLX_REG(a)	"PLX " #a, GSC_PLX_9080_##a



// data types *****************************************************************

typedef struct
{
	const char*	name;
	U32			reg;
} _reg_def_t;



// variables ******************************************************************

static const _reg_def_t	_gsc[]	=
{
	{ _GSC_REG(FRR)		},
	{ _GSC_REG(BCR)		},
	{ _GSC_REG(BSR)		},
	{ _GSC_REG(TAR)		},
	{ _GSC_REG(RAR)		},
	{ _GSC_REG(FSR)		},
	{ _GSC_REG(FDR)		},
	{ _GSC_REG(TSVLCR)	},
	{ _GSC_REG(TLVLCR)	},
	{ _GSC_REG(TLILCR)	},
	{ _GSC_REG(RSCR)	},
	{ _GSC_REG(RLCR)	},
	{ _GSC_REG(ICR)		},
	{ _GSC_REG(ISR)		},
	{ _GSC_REG(TCDR)	},
	{ _GSC_REG(TFSR)	},
	{ _GSC_REG(RFSR)	},
	{ _GSC_REG(TFWR)	},
	{ _GSC_REG(RFWR)	},
	{ _GSC_REG(IELR)	},
	{ _GSC_REG(IHLR)	},
	{ _GSC_REG(ACR)		},
	{ _GSC_REG(GOR)		},
	{ _GSC_REG(GIR)		},
	{ NULL, 0			}
};

static const _reg_def_t	_pci[]	=
{
	// PCI9080
	{ _PCI_REG(VIDR)		},
	{ _PCI_REG(DIDR)		},
	{ _PCI_REG(CR)			},
	{ _PCI_REG(SR)			},
	{ _PCI_REG(REV)			},
	{ _PCI_REG(CCR)			},
	{ _PCI_REG(CLSR)		},
	{ _PCI_REG(LTR)			},
	{ _PCI_REG(HTR)			},
	{ _PCI_REG(BISTR)		},
	{ _PCI_REG(BAR0)		},
	{ _PCI_REG(BAR1)		},
	{ _PCI_REG(BAR2)		},
	{ _PCI_REG(BAR3)		},
	{ _PCI_REG(BAR4)		},
	{ _PCI_REG(BAR5)		},
	{ _PCI_REG(CIS)			},
	{ _PCI_REG(SVID)		},
	{ _PCI_REG(SID)			},
	{ _PCI_REG(ERBAR)		},
	{ _PCI_REG(ILR)			},
	{ _PCI_REG(IPR)			},
	{ _PCI_REG(MGR)			},
	{ _PCI_REG(MLR)			},
	{ NULL, 0				}
};

static const _reg_def_t	_plx[]	=
{
	// Local Configuration Registers: PCI9080
	{ _PLX_REG(LAS0RR)		},
	{ _PLX_REG(LAS0BA)		},
	{ _PLX_REG(MARBR)		},
	{ _PLX_REG(BIGEND)		},
	{ _PLX_REG(EROMRR)		},
	{ _PLX_REG(EROMBA)		},
	{ _PLX_REG(LBRD0)		},
	{ _PLX_REG(DMRR)		},
	{ _PLX_REG(DMLBAM)		},
	{ _PLX_REG(DMLBAI)		},
	{ _PLX_REG(DMPBAM)		},
	{ _PLX_REG(DMCFGA)		},
	{ _PLX_REG(LAS1RR)		},
	{ _PLX_REG(LAS1BA)		},
	{ _PLX_REG(LBRD1)		},

	// Runtime Registers: PCI9080
	{ _PLX_REG(MBOX0)		},
	{ _PLX_REG(MBOX1)		},
	{ _PLX_REG(MBOX2)		},
	{ _PLX_REG(MBOX3)		},
	{ _PLX_REG(MBOX4)		},
	{ _PLX_REG(MBOX5)		},
	{ _PLX_REG(MBOX6)		},
	{ _PLX_REG(MBOX7)		},
	{ _PLX_REG(P2LDBELL)	},
	{ _PLX_REG(L2PDBELL)	},
	{ _PLX_REG(INTCSR)		},
	{ _PLX_REG(CNTRL)		},
	{ _PLX_REG(PCIHIDR)		},
	{ _PLX_REG(PCIHREV)		},

	// DMA Registers: PCI9080
	{ _PLX_REG(DMAMODE0)	},
	{ _PLX_REG(DMAPADR0)	},
	{ _PLX_REG(DMALADR0)	},
	{ _PLX_REG(DMASIZ0)		},
	{ _PLX_REG(DMADPR0)		},
	{ _PLX_REG(DMACSR0)		},
	{ _PLX_REG(DMAMODE1)	},
	{ _PLX_REG(DMAPADR1)	},
	{ _PLX_REG(DMALADR1)	},
	{ _PLX_REG(DMASIZ1)		},
	{ _PLX_REG(DMADPR1)		},
	{ _PLX_REG(DMACSR1)		},
	{ _PLX_REG(DMAARB)		},
	{ _PLX_REG(DMATHR)		},

	// Messaging Queue Registers: PCI9080
	{ _PLX_REG(OPLFIS)		},
	{ _PLX_REG(OPLFIM)		},
	{ _PLX_REG(IQP)			},
	{ _PLX_REG(OQP)			},
	{ _PLX_REG(MQCR)		},
	{ _PLX_REG(QBAR)		},
	{ _PLX_REG(IFHPR)		},
	{ _PLX_REG(IFTPR)		},
	{ _PLX_REG(IPHPR)		},
	{ _PLX_REG(IPTPR)		},
	{ _PLX_REG(OFHPR)		},
	{ _PLX_REG(OFTPR)		},
	{ _PLX_REG(OPHPR)		},
	{ _PLX_REG(OPTPR)		},
	{ _PLX_REG(QSR)			},

	{ NULL, 0				}
};



/******************************************************************************
*
*	Function:	_reg_list
*
*	Purpose:
*
*		List the given registers and their values.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*		list	The register list to display.
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

static int _reg_list(void* dev, const _reg_def_t* list)
{
	int	errs	= 0;
	int	status;
	U32	v;

	printf("\n\n");

	for (; list->name; list++)
	{
		status	= hpdi32_reg_read(dev, list->reg, &v);
		printf("%-10s 0x%08lX", list->name, (long) v);

		if (status)
		{
			errs++;
			printf("  FAIL <---  (hpdi32_reg_read()");
		}

		printf("\n");
	}

	return(errs);
}



/******************************************************************************
*
*	Function:	_reg_name
*
*	Purpose:
*
*		Return the name of the given register.
*
*	Arguments:
*
*		reg		The register id.
*
*		list	The list of registers to check.
*
*	Returned:
*
*		!NULL	The name of the given register.
*		NULL	The register id isn't in the given list.
*
******************************************************************************/

static const char* _reg_name(U32 reg, const _reg_def_t* list)
{
	const char*	ptr	= NULL;

	for (; list->name; list++)
	{
		if (list->reg == reg)
		{
			ptr	= list->name;
			break;
		}
	}

	return(ptr);
}



/******************************************************************************
*
*	Function:	dev_reg_mod
*
*	Purpose:
*
*		Put a wrapper around hpdi32_reg_mod().
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*		reg		The register to access.
*
*		value	The value to apply.
*
*		mask	The set of bits to modify.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dev_reg_mod(void* dev, U32 reg, U32 value, U32 mask, int verbose)
{
	int	errs;
	U32	status;

	status	= hpdi32_reg_mod(dev, reg, value, mask);
	errs	= label_fail_func(verbose, "hpdi32_reg_mod", status);
	return(errs);
}



/******************************************************************************
*
*	Function:	dev_reg_read
*
*	Purpose:
*
*		Put a wrapper around hpdi32_reg_read().
*
*	Arguments:
*
*		dev		The handle for the device of interest.
*
*		reg		The register to access.
*
*		value	The value read goes here.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dev_reg_read(void* dev, U32 reg, U32* value, int verbose)
{
	int	errs;
	U32	status;

	status	= hpdi32_reg_read(dev, reg, value);
	errs	= label_fail_func(verbose, "hpdi32_reg_read", status);
	return(errs);
}



/******************************************************************************
*
*	Function:	dev_reg_write
*
*	Purpose:
*
*		Put a wrapper around the hpdi32_reg_write() call.
*
*	Arguments:
*
*		dev		The handle for the device to access.
*
*		reg		The register to access.
*
*		value	The value to write.
*
*		verbose	Work verbosely?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int dev_reg_write(void* dev, U32 reg, U32 value, int verbose)
{
	int	errs;
	U32	status;

	status	= hpdi32_reg_write(dev, reg, value);
	errs	= label_fail_func(verbose, "hpdi32_reg_write", status);
	return(errs);
}



/******************************************************************************
*
*	Function:	reg_list
*
*	Purpose:
*
*		List the given registers and their values.
*
*	Arguments:
*
*		dev		The handle of the device to access.
*
*		pci		List the PCI registers?
*
*		plx		List the PLX registers?
*
*		gsc		List the GSC registers?
*
*	Returned:
*
*		>= 0	The number of errors encountered here.
*
******************************************************************************/

int reg_list(void* dev, int pci, int plx, int gsc)
{
	int	errs	= 0;

	if (pci)
		errs	+= _reg_list(dev, _pci);

	if (plx)
		errs	+= _reg_list(dev, _plx);

	if (gsc)
		errs	+= _reg_list(dev, _gsc);

	return(errs);
}



/******************************************************************************
*
*	Function:	reg_name
*
*	Purpose:
*
*		Return the name of the given register.
*
*	Arguments:
*
*		reg		The register id.
*
*	Returned:
*
*		!NULL	Either the name or an error string.
*
******************************************************************************/

const char* reg_name(U32 reg)
{
	static	char	buf[32];
	const char*		ptr;

	ptr	= _reg_name(reg, _gsc);

	if (ptr == NULL)
		ptr	= _reg_name(reg, _pci);

	if (ptr == NULL)
		ptr	= _reg_name(reg, _plx);

	if (ptr == NULL)
	{
		sprintf(buf, "0x%lX", (long) reg);
		ptr	= buf;
	}

	return(ptr);
}


