// $URL: http://subversion:8080/svn/gsc/trunk/drivers/LINUX/HPDI32/HPDI32_Linux_sdk_6.x.x_GSC_DN/xfer/main.c $
// $Rev: 30446 $
// $Date: 2014-10-07 16:57:57 -0500 (Tue, 07 Oct 2014) $

#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/timeb.h>

#include "main.h"



// variables ******************************************************************

		U32		block_count		= 100L * 1024 * 1024;
		U32		block_samples	= 256;
		U32		io_clock		= 20000000L;

static	char	_config_file[256];
static	int		_continuous		= 0;
static	int		_def_board		= -1;
static	int		_ignore_errors	= 0;
static	long	_minute_limit	= 0;
static	int		_quit			= 0;
static	int		_receive		= 0;
static	int		_test_limit		= -1;
static	int		_transmit		= 0;



/******************************************************************************
*
*	Function:	_u32_2_text_k
*
*	Purpose:
*
*		Format a value that is evently divisable by a "k" sized unit into a
*		string.
*
*	Arguments:
*
*		u32		The value to convert.
*
*		dest	The destination buffer. This is presumed to be large enough.
*
*		k		The number of values in a "1K" unit.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _u32_2_text_k(U32 u32, char* dest, U32* k)
{
	char*	approx	= "";
	char*	factor	= "";

	if ((u32 % k[0]) == 0)
	{
		factor	= "K";
		u32		= u32 / k[0];
	}
	else if (u32 > ((U32) 10L * k[0]))
	{
		approx	= "~ ";
		factor	= "K";
		u32		= (u32 + (k[0] / 2)) / k[0];
	}

	if ((u32 % k[0]) == 0)
	{
		factor	= "M";
		u32		= u32 / k[0];
	}
	else if (u32 > (10 * k[0]))
	{
		approx	= "~ ";
		factor	= "M";
		u32		= (u32 + (k[0] / 2)) / k[0];
	}

	if ((u32 % k[0]) == 0)
	{
		factor	= "G";
		u32		= u32 / k[0];
	}
	else if (u32 > (10L * k[0]))
	{
		approx	= "~ ";
		factor	= "G";
		u32		= (u32 + (k[0] / 2)) / k[0];
	}

	sprintf(dest, "%s%lu%s", approx, (long) u32, factor);

	if (factor[0] == 0)
		k[0]	= 0;
}



/******************************************************************************
*
*	Function:	_u32_2_text
*
*	Purpose:
*
*		Format a value into a string.
*
*	Arguments:
*
*		u32		The value to convert.
*
*		dest	The destination buffer.
*
*		size	The size of the destination buffer.
*
*		k		The size of a K is recorded here. If a factor is not used
*				then this will be zero.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _u32_2_text(U32 u32, char* dest, size_t size, U32* k)
{
	int		a;
	int		b;
	char	buf[64];

	a	= u32 % 1000;
	b	= u32 % 1024;

	if (u32 < 1000)
	{
		k[0]	= 0;
		sprintf(buf, "%ld", (long) u32);
	}
	else if (a <= b)
	{
		k[0]	= 1000;
		_u32_2_text_k(u32, buf, k);
	}
	else
	{
		k[0]	= 1024;
		_u32_2_text_k(u32, buf, k);
	}

	strncpy(dest, buf, size);
	dest[size - 1]	= 0;
}



/******************************************************************************
*
*	Function:	_parse_args
*
*	Purpose:
*
*		Parse the command line arguments.
*
*	Arguments:
*
*		dev		The handle of the board to access.
*
*	Returned:
*
*		None.
*
******************************************************************************/

static void _parse_args(int argc, char** argv)
{
	char	c;
	FILE*	file;
	int		i;
	int		j;
	int		k;

	printf("USAGE: xfer <-c> <-C> <-m#> <-n#> <index> [tx|rx] [file]\n");
	printf("  -c     Continuous testing, but stop on first error.\n");
	printf("  -C     Continuous testing, disregard errors.\n");
	printf("  -m#    Run for at most # minutes (a decimal number).\n");
	printf("  -n#    Repeat test at most # times (a decimal number).\n");
	printf("  index  The zero based index of the board to access.\n");
	printf("  Tx|Rx  Transmit or receive data.\n");
	printf("  file   Configuration data file.\n");
	printf("  NOTE:  Hit Ctrl-C to interrupt continuous testing.\n");

	for (i = 1; i < argc; i++)
	{
		if (strcmp(argv[i], "-c") == 0)
		{
			_continuous	= 1;
			_ignore_errors	= 0;
			continue;
		}

		if (strcmp(argv[i], "-C") == 0)
		{
			_continuous	= 1;
			_ignore_errors	= 1;
			continue;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'm') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_minute_limit	= k;
				continue;
			}

			_quit	= 1;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			break;
		}

		if ((argv[i][0] == '-') && (argv[i][1] == 'n') && (argv[i][2]))
		{
			j	= sscanf(&argv[i][2], "%d%c", &k, &c);

			if ((j == 1) && (k > 0))
			{
				_test_limit	= k;
				continue;
			}

			_quit	= 1;
			printf("ERROR: invalid argument: %s\n", argv[i]);
			break;
		}

		if (stricmp(argv[i], "rx") == 0)
		{
			_receive	= 1;
			_transmit	= 0;
			continue;
		}

		if (stricmp(argv[i], "tx") == 0)
		{
			_transmit	= 1;
			_receive	= 0;
			continue;
		}

		if (isdigit(argv[i][0]))
		{
			j	= sscanf(argv[i], "%d%c", &k, &c);

			if ((j == 1) && (k >= 0))
			{
				_def_board	= k;
				continue;
			}
		}

		file	= fopen(argv[i], "rb");

		if (file)
		{
			fclose(file);
			strcpy(_config_file, argv[i]);
			continue;
		}

		_quit	= 1;
		printf("ERROR: can't access configuration file: '%s'\n", argv[i]);
		break;
	}

	if ((_quit == 0) && (_transmit == 0) && (_receive == 0))
	{
		_quit	= 1;
		printf("ERROR: invalid data direction: none specified\n");
	}

	if ((_quit == 0) && (_config_file[0] == 0))
	{
		_quit	= 1;
		printf("ERROR: configuration file not specified\n");
	}
}



/******************************************************************************
*
*	Function:	_perform_tests
*
*	Purpose:
*
*		Perform the appropriate testing.
*
*	Arguments:
*
*		dev		The handle of the board to access.
*
*		index	The index of the above board.
*
*	Returned:
*
*		>= 0	The number of errors seen.
*
******************************************************************************/

static int _perform_tests(void* dev, int index)
{
	int			errs	= 0;
	char*		psz;
	U32			status;
	struct tm*	stm;
	time_t		tt;

	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	label_text("Performing Tests");
	printf("%s", psz);

	id_host();
	errs	+= id_software(dev, VERSION);
	errs	+= id_hardware(dev, index);

	status	= read_config_file(dev, _config_file);

	if (status != GSC_SUCCESS)
		;
	else if (_transmit)
		status	= tx_process(dev);
	else if (_receive)
		status	= rx_process(dev);

	errs	+= (status == GSC_SUCCESS) ? 0 : 1;

	return(errs);
}



/******************************************************************************
*
*	Function:	actual_duration
*
*	Purpose:
*
*		Report the actual duration of the transfer.
*
*	Arguments:
*
*		begin	The time the transfer began.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void actual_duration(long begin)
{
	long		days;
	long		end;
	long		hours;
	long		mins;
	const char*	psz;
	long		secs;
	struct tm*	stm;
	time_t		tt;

	label_text("End Time");
	end		= (long) time_ms_get_run();
	time(&tt);
	stm	= localtime(&tt);
	psz	= asctime(stm);
	printf("%s", psz);

	secs	= (end - begin + 500) / 1000;
	days	= secs / SECONDS_PER_DAY;
	secs	= secs % SECONDS_PER_DAY;
	hours	= secs / SECONDS_PER_HOUR;
	secs	= secs % SECONDS_PER_HOUR;
	mins	= secs / SECONDS_PER_MINUTE;
	secs	= secs % SECONDS_PER_MINUTE;
	label_text("Duration");

	if (days)
		printf("%ld day%s, ", days, (days == 1) ? "" : "s");

	if ((days) || (hours))
		printf("%ld hour%s, ", hours, (hours == 1) ? "" : "s");

	if ((days) || (hours) || (mins))
		printf("%ld minute%s, ", mins, (mins == 1) ? "" : "s");

	printf("%ld second%s\n", secs, (secs == 1) ? "" : "s");
}



/******************************************************************************
*
*	Function:	estimate_duration
*
*	Purpose:
*
*		Show the estimated duration for the transfer.
*
*	Arguments:
*
*		clk			The cable clock rate.
*
*		size		The data size in bytes is recorded here.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 estimate_duration(U32 clk, U32 size, double bytes)
{
	double	days;
	double	hours;
	double	minutes;
	double	seconds;
	U32		status	= GSC_SUCCESS;
	double	years;
	double	weeks;

	for (;;)
	{
		printf("Estimated Time: ");
		seconds	= bytes / size / clk;

		years	= (long) (seconds / SECONDS_PER_YEAR);
		seconds	= seconds - (years * SECONDS_PER_YEAR);

		weeks	= (long) (seconds / SECONDS_PER_WEEK);
		seconds	= (long) seconds - (weeks * SECONDS_PER_WEEK);

		days	= (long) (seconds / SECONDS_PER_DAY);
		seconds	= (long) seconds - (days * SECONDS_PER_DAY);

		hours	= (long) (seconds / SECONDS_PER_HOUR);
		seconds	= (long) seconds - (hours * SECONDS_PER_HOUR);

		minutes	= (long) (seconds / SECONDS_PER_MINUTE);
		seconds	= (long) seconds - (minutes * SECONDS_PER_MINUTE);

		if (years > 10)
		{
			printf("over %lu years", (long) years);
		}
		else
		{
			printf("~ ");

			if (years >= 1)
				printf("%lu year%s, ", (long) years, (years == 1) ? "" : "s");

			if ((years >= 1) || (weeks >= 1))
				printf("%lu week%s, ", (long) weeks, (weeks == 1) ? "" : "s");

			if ((years >= 1) || (weeks >= 1) || (days >= 1))
				printf("%lu day%s, ", (long) days, (days == 1) ? "" : "s");

			if ((years >= 1) || (weeks >= 1) || (days >= 1) || (hours >= 1))
				printf("%lu hour%s, ", (long) hours, (hours == 1) ? "" : "s");

			if ((years >= 1) || (weeks >= 1) || (days >= 1) || (hours >= 1) || (minutes >= 1))
				printf("%lu minute%s, ", (long) minutes, (minutes == 1) ? "" : "s");

			printf("%lu second%s", (long) seconds, (seconds == 1) ? "" : "s");
		}

		if ((years >= 1) || (weeks >= 1) || (days > TRANSFER_PERIOD_LIMIT_DAYS))
		{
			printf(	"\nERROR: duration is too long. The limit is %d days.\n",
					(int) TRANSFER_PERIOD_LIMIT_DAYS);
			status	= GSC_FAILED;
			break;
		}

		printf("\n");
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	format_value
*
*	Purpose:
*
*		Print the given value using K, M and G, as appropriate.
*
*	Arguments:
*
*		u32		The value to print.
*
*		units	The units to display after the value.
*
*	Returned:
*
*		None.
*
******************************************************************************/

void format_value(U32 u32, const char* units)
{
	char	buf[64];
	U32		k;
	char	kbuf[64];

	_u32_2_text(u32, buf, sizeof(buf), &k);

	if (k)
		sprintf(kbuf, " (1K = %lu)", (long) k);
	else
		kbuf[0]	= 0;

	printf("%s%s%s", buf, units, kbuf);
}



/******************************************************************************
*
*	Function:	show_data_rate
*
*	Purpose:
*
*		Show the maximum data transfer rate.
*
*	Arguments:
*
*		which		The appropriate Tx or Rx "which" bit.
*
*		clk			The clock rate at the cable.
*
*		size		The data size in bytes.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 show_data_rate(U32 which, U32 clk, U32 size)
{
	U32	rate;

	printf("%s Data rate:   ", (which == HPDI32_WHICH_TX) ? "Tx" : "Rx");
	rate	= clk * size;
	printf("%lu * %lu = ", (long) clk, (long) size);
	format_value(rate, "B/S (maximum)");
	printf("\n");
	return(GSC_SUCCESS);
}



/******************************************************************************
*
*	Function:	show_data_set
*
*	Purpose:
*
*		Show the data set to use.
*
*	Arguments:
*
*		device		The handle used to access the device under test.
*
*		which		The appropriate Tx or Rx "which" bit.
*
*		size		The data size in bytes is recorded here.
*
*		bytes		The number of bytes is stored her.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 show_data_set(void* device, U32 which, unsigned long* size, double* bytes)
{
	double	d;
	U32		status;

	for (;;)
	{
		printf("%s Data Set:    ", (which == HPDI32_WHICH_TX) ? "Tx" : "Rx");
		status	= HPDI32_IO_DATA_SIZE__GET(device, which, size);
		test_status(status, "HPDI32_IO_DATA_SIZE__GET");
		if (status != GSC_SUCCESS)	break;

		if (size[0] == HPDI32_IO_DATA_SIZE_8_BITS)
			size[0]	= 1;
		else if (size[0] == HPDI32_IO_DATA_SIZE_16_BITS)
			size[0]	= 2;
		else
			size[0]	= 4;

		printf(	"%lu * %lu * %lu = ",
				(long) size[0],
				(long) block_samples,
				(long) block_count);

		if (block_samples > BLOCK_SAMPLES_LIMIT)
		{
			printf("\nERROR: BlockSamples is too large. The limit is ");
			format_value(BLOCK_SAMPLES_LIMIT, ".\n");
			status	= GSC_FAILED;
			break;
		}

		if (block_count > BLOCK_COUNT_LIMIT)
		{
			printf("\nERROR: BlockCount is too large. The limit is ");
			format_value(BLOCK_COUNT_LIMIT, ".\n");
			status	= GSC_FAILED;
			break;
		}

		bytes[0]	= (double) size[0] * block_samples * block_count;

		if (bytes[0] > (double) 0xFFFFFFFFUL)
		{
			d	= bytes[0];
			d	= d / 1000;		// K
			d	= d / 1000;		// M
			d	= d / 1000;		// G
			printf("~ %.0fGB (1K = 1000)", (float) d);
		}
		else
		{
			format_value((U32) bytes[0], "B");
		}

		printf("\n");
		break;
	}

	return(status);
}



/******************************************************************************
*
*	Function:	tx_rx_tristate
*
*	Purpose:
*
*		Enable the Tx/Rx Tristate parameter as we may have two HPDI32s
*		talking back-to-back.
*
*	Arguments:
*
*		device		The handle used to access the device.
*
*	Returned:
*
*		GSC_SUCCESS	All went well.
*		else		An appropriate error status.
*
******************************************************************************/

U32 tx_rx_tristate(void* device)
{
	U32	status;

	status	= HPDI32_MISC_TX_RX_TRI_STATE__YES(device);
	test_status(status, "HPDI32_MISC_TX_RX_TRI_STATE__YES");
	return(status);
}



/******************************************************************************
*
*	Function:	test_status
*
*	Purpose:
*
*		See if the given status is OK. If it is then return zero. If not, then
*		print an error message and return non-zero.
*
*	Arguments:
*
*		status		The status being tested.
*
*		service		The API service that produced the above status.
*
*	Returned:
*
*		0			All went well.
*		else		There was a problem.
*
******************************************************************************/

U32 test_status(U32 status, const char* service)
{
	if (status == GSC_SUCCESS)
		status	= 0;
	else
		printf("ERROR: %s returned 0x%lX\n", service, (long) status);

	return(status);
}



/******************************************************************************
*
*	Function:	main
*
*	Purpose:
*
*		Program entry point.
*
*	Arguments:
*
*		argc			The number of command line arguments.
*
*		argv			The list of command line arguments.
*
*	Returned:
*
*		EXIT_SUCCESS	No errors were seen.
*		EXIT_FAILURE	Errors were seen.
*
******************************************************************************/

int main(int argc, char** argv)
{
	void*	dev			= NULL;
	int		errs		= 0;
	time_t	exec		= time(NULL);
	long	failures	= 0;
	long	hours;
	long	mins;
	time_t	now;
	long	passes		= 0;
	int		qty;
	long	secs;
	time_t	test;
	long	tests		= 0;
	time_t	t_limit;

	for (;;)
	{
		test	= time(NULL);
		label_init(28);
		_parse_args(argc, argv);

		if (_quit)
			break;

		printf("\n");
		t_limit	= exec + (_minute_limit * 60);
		errs	= dev_api_status();

		if (errs == 0)
		{
			qty	= dev_board_count();
			errs	= select_1_board(qty, &_def_board);
		}

		if (errs == 0)
		{
			label_text("Testing Board Index");
			printf("%d\n", _def_board);
			errs	= dev_open((U8) _def_board, &dev);
		}

		if (errs == 0)
		{
			errs	= _perform_tests(dev, _def_board);
			errs	+= dev_close((U8) _def_board, dev);
		}

		now	= time(NULL);
		tests++;

		if (errs)
		{
			failures++;
			printf("\nRESULTS: FAIL <---");
		}
		else
		{
			passes++;
			printf("\nRESULTS: PASS");
		}

		secs	= now - test;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(" (duration %ld:%ld:%02ld)\n", hours, mins, secs);

		secs	= now - exec;
		hours	= secs / 3600;
		secs	= secs % 3600;
		mins	= secs / 60;
		secs	= secs % 60;
		printf(	"SUMMARY: tests %ld, pass %ld, fail %ld"
				" (duration %ld:%ld:%02ld)\n\n",
				tests,
				passes,
				failures,
				hours,
				mins,
				secs);

		if ((_test_limit > 0) && (_test_limit <= tests))
			break;

		if (_continuous == 0)
			break;

		if ((_ignore_errors == 0) && (errs))
			break;

		if ((_minute_limit) && (now >= t_limit))
			break;
	}

	errs	= errs ? EXIT_FAILURE : EXIT_SUCCESS;
	return(errs);
}



//******************************************************************************
U32 my_mem_alloc(void* device, U32 which, U32 size, void** ptr)
{
	unsigned long	get	= 0;
	U32				sts;
	void*			vp;

	if (ptr == NULL)
		ptr	= &vp;

	sts	= HPDI32_IO_BUFFER_SIZE__SET(device, which, size, &get);

	if (sts != GSC_SUCCESS)
	{
		ptr[0]	= NULL;
	}
	else if (get < size)
	{
		sts		= GSC_FAILED;
		ptr[0]	= NULL;
	}
	else if (size)
	{
		sts		= HPDI32_IO_BUFFER_POINTER__GET(device, which, &get);
		ptr[0]	= (void*) get;
	}
	else
	{
		ptr[0]	= NULL;
	}

	return(sts);
}



//******************************************************************************
void my_mem_free(void* device, U32 which)
{
	my_mem_alloc(device, which, 0, NULL);
}



